﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Web.UI.WebControls;
using EmpCRM.common;
using EmpCRM.Dto;

namespace EmpCRM
{
    /// <summary>
    /// 選任/解任申請リスト出力クラス
    /// </summary>
    public partial class ApplicationListPrint : System.Web.UI.Page
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        //共通クラス
        private ComFunc m_ComFunc = new ComFunc();

        //GridViewページサイズ
        private static int gvPagesize = int.Parse(ConfigurationManager.AppSettings[ComConst.CONFIG_GRIDVIEW_PAGESIZE]);

        //画面項目
        //出力ファイル名(パス？）
        string strOutputFileName = string.Empty;

        //シート名
        string strSheetName = string.Empty;
        //提出先総通局
        static string strSotsuKyoku = string.Empty;
        //所属会社
        string strCompany = string.Empty;
        //所属部署
        string strDepartment = string.Empty;
        //JMCIAフラグ
        int? jmciaFlag = null;
        //SB/WCPフラグ
        int? sbwcpFlag = null;
        //地域名
        private static string areaName = string.Empty;

        private string[] areaRowNames = { "hokkaidou", "touhoku", "hokuriku", "shinetsu", "kantou", "tokai", "kinki", "shikoku", "chugoku", "kyuushu", "okinawa" };

        #region イベント定義
        protected void Page_Load(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //ログイン状態チェック
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                lblErrMessage.Text = string.Empty;

                //開始ログ出力
                logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "Page_Load");
                if (!IsPostBack)
                {
                    //初期表示時
                    try
                    {
                        try
                        {
                            Dictionary<string,string> dictBumon = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string,string>;

                            //システム管理者の場合
                            if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
                            {
                                //システム部門専用検索項目を表示
                                pnlSystemUserSearchCondition.Visible = true;

                                //管理部門DDL生成
                                //空項目を追加
                                //ddlBumon.Items.Add(new ListItem(string.Empty, string.Empty));

                                ////現在ADに存在する部門一覧をDDLに設定
                                //foreach (KeyValuePair<string, string> pair in dictBumon)
                                //{
                                //    ddlBumon.Items.Add(new ListItem(pair.Value, pair.Key));
                                //}
                                //ddlBumon.Visible = true;
                                ccDepartmentSelector.Visible = true;
                            }
                            else //システム管理者でない場合
                            {
                                if (dictBumon.ContainsKey(userRole.DepartmentId))
                                {
                                    lblBumon.Text = dictBumon[userRole.DepartmentId];
                                }
                                else
                                {
                                    lblBumon.Text = userRole.DepartmentId;
                                }
                                lblBumon.Visible = true;
                            }

                            //JMCIA
                            ddlJmcia.Items.Add(new ListItem(string.Empty, string.Empty));
                            ddlJmcia.Items.Add(new ListItem("OFF", "0"));
                            ddlJmcia.Items.Add(new ListItem("ON", "1"));

                            //SB/WCP
                            //マスタからSB/WCPの値を取得
                            var lstSbwcp = m_ComFunc.GetSbWcpList();
                            ddlSbwcp.Items.Add(new ListItem(string.Empty, string.Empty));
                            foreach (KeyValuePair<string, string> pair in lstSbwcp)
                            {
                                ddlSbwcp.Items.Add(new ListItem(pair.Value, pair.Key));
                            }

                            createDdlSotsukyokuList(false);
                        }
                        catch (Exception ex)
                        {
                            logger.Error("総合通信局取得失敗", ex);
                            lblErrMessage.Text += " 総合通信局取得失敗 ";
                        }

                        try
                        {
                            //所属会社DDL生成
                            m_ComFunc.SetBelongCompanyDropdownlist(ddlCompany);
                        }
                        catch(Exception ex)
                        {
                            logger.Error("所属会社取得失敗", ex);
                            lblErrMessage.Text += " 所属会社取得失敗 ";

                        }
                    }
                    catch (Exception ex)
                    {
                        logger.Error("User:" + userRole.UserId + ex.Message, ex);
                        lblErrMessage.Text += " 初期表示処理不正終了 ";
                    }
                    //所属部署DDL生成
                    //初期状態は空白にしておき、所属会社を選んだら自動でDBに取得しにいく
                    ddlDepartment.Items.Add(new ListItem(string.Empty, string.Empty));

                    //デフォルトファイル名を設定
                    txtDownloadFileName.Text = ConfigurationManager.AppSettings[ComConst.CONFIG_APPLICATION_LIST_FILE_NAME];
                }
                else
                {
                    //Postback時
                }
                //終了ログ出力
                logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "Page_Load");
            }
        }

        /// <summary>
        /// 画面で選択された会社IDを元に所属部署を取得し表示する。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddlCompany_Changed(object sender, EventArgs e)
        {
            //選択された会社に対応する部署を設定
            m_ComFunc.SetBelongDepartmentDropdownlist(ddlDepartment, ddlCompany.SelectedValue.ToString());
        }

        //検索ボタン押下時処理
        protected void BtnSearch_Click(object sender, EventArgs e)
        {
            UserRole user = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + user.UserId + "検索ボタン押下処理");

            lblErrMessage.Text = string.Empty;
            //検索結果クリア
            Session["RESULT"] = null;

            //画面項目の値を取得

            //シート名
            strSheetName = txtSheetName.Text;

            //提出先総通局
            strSotsuKyoku = ddlSotsu.SelectedValue;

            //所属会社
            strCompany = ddlCompany.SelectedItem.Text;
            //所属部署
            if (ddlDepartment.SelectedItem != null)
            {
                strDepartment = ddlDepartment.SelectedItem.Text;
            }
            else
            {
                strDepartment = string.Empty;
            }
            //JMCIAフラグ取得処理
            if (string.IsNullOrEmpty(ddlJmcia.SelectedValue.ToString()))
            {
                jmciaFlag = null;
            }
            else
            {
                jmciaFlag = int.Parse(ddlJmcia.SelectedValue.ToString());
            }

            //sb/wcpフラグ取得
            if (string.IsNullOrEmpty(ddlSbwcp.SelectedValue.ToString()))
            {
                sbwcpFlag = null;
            }
            else
            {
                sbwcpFlag = int.Parse(ddlSbwcp.SelectedValue.ToString());
            }

            //総通局列用の定数値
            string areaColName = "thcAreaName";

            if (!string.IsNullOrEmpty(strSotsuKyoku))
            {
                //相通局種別の見出しを動的に設定する。
                areaName = m_ComFunc.GetAreaName(strSotsuKyoku);
                gvApplicationTarget.Columns[12].HeaderText = areaName;
            }


            //地域ごとが選択されている場合
            if (radioRegional.Checked)
            {
                radioBumon.Enabled = false;
                //従来の検索処理を実行

                List<SqlParameter> lstParam = new List<SqlParameter>();

                //申請対象リスト検索処理
                string strSQL = string.Empty;
                strSQL += "SELECT";
                strSQL += " ROW_NUMBER() OVER (ORDER BY psn.enginnerID,psn.selectionID) AS 'No'";
                strSQL += " , eng.LastName AS '氏名（姓）'";
                strSQL += " , eng.FirstName AS '氏名（名）'";
                strSQL += " , eng.LastName_Kana AS 'ふりがな（姓）'";
                strSQL += " , eng.FirstName_Kana AS 'ふりがな（名）'";
                strSQL += " ,eng.license_number AS '免許番号'";
                strSQL += ", case psn.jmcia_flag  when 0 then '' when 1 then '●' end as 'JMCIA'";
                //strSQL += " , case psn.sb_wcp  when 0 then 'SB' when 1 then 'WCP' when 2 then 'SB/WCP' end AS 'SB/WCP'";
                strSQL += " , psn.sb_wcp ";
                strSQL += " ,psn.belong_company AS '所属会社'";
                strSQL += " ,date1.sr_date AS '選任日'";
                strSQL += " ,date2.sr_date AS '解任日'";


                if (!string.IsNullOrEmpty(strSotsuKyoku))
                {
                    strSQL += " ,psn." + m_ComFunc.GetAreaRowName(strSotsuKyoku) + " AS '" + areaColName + "'";
                }
                else
                {
                    strSQL += ",''" + areaColName;
                }

                strSQL += " FROM selection_personal_tbl psn";
                strSQL += " INNER JOIN radio_enginner_info_tbl eng";
                strSQL += " ON eng.enginnerID = psn.enginnerID";

                strSQL += " LEFT OUTER JOIN selection_date_tbl date1";
                strSQL += " ON date1.enginnerID = psn.enginnerID";
                strSQL += " AND date1.selectionID = psn.selectionID";
                strSQL += " AND date1.request_date = psn.request_date";
                strSQL += " AND date1.sr_flag = 1";

                strSQL += " LEFT OUTER JOIN selection_date_tbl date2";
                strSQL += " ON date2.enginnerID = psn.enginnerID";
                strSQL += " AND date2.selectionID = psn.selectionID";
                strSQL += " AND date2.request_date = psn.request_date";
                strSQL += " AND date2.sr_flag = 2";
                strSQL += " AND date2.area_code = date1.area_code";

                //Where句
                strSQL += " WHERE psn.delete_flag = 0";

                // 承認済みのデータのみを表示
                strSQL += " AND psn.apploval_flag = 2";

                //所属会社
                if (!string.IsNullOrEmpty(strCompany))
                {
                    strSQL += " AND psn.belong_company like @company";
                    //パラメータ設定
                    lstParam.Add(new SqlParameter("@company", "%" + strCompany + "%"));
                }

                //所属部署
                if (!string.IsNullOrEmpty(strDepartment))
                {
                    strSQL += " AND psn.belong_department like @department";
                    lstParam.Add(new SqlParameter("@department", "%" + strDepartment + "%"));
                }

                //JMCIAフラグ
                if (jmciaFlag != null)
                {
                    strSQL += " AND psn.jmcia_flag = @jmcia";
                    lstParam.Add(new SqlParameter("@jmcia", jmciaFlag));
                }

                //SBWCPフラグ
                if (sbwcpFlag != null)
                {
                    strSQL += " AND psn.sb_wcp = @sbwcp";
                    lstParam.Add(new SqlParameter("@sbwcp", sbwcpFlag));
                }

                //提出先総通局
                if (!string.IsNullOrEmpty(strSotsuKyoku))
                {
                    int areaCode = m_ComFunc.getAreaCode(strSotsuKyoku);
                    string rowName = m_ComFunc.GetAreaRowName(strSotsuKyoku);

                    strSQL += " AND psn." + rowName + " IS NOT NULL AND LEN(psn." + rowName + ") > 0";
                    if (chkEX.Checked) strSQL += " AND psn." + rowName + " like '%EX%'";
                    if (chkFX.Checked) strSQL += " AND psn." + rowName + " like '%FX%'";
                    if (chkFB.Checked) strSQL += " AND psn." + rowName + " like '%FB%'";
                    if (chkFBR.Checked) strSQL += " AND psn." + rowName + " like '%FBR%'";
                    if (chkML.Checked) strSQL += " AND psn." + rowName + " like '%ML%'";

                    strSQL += " AND date1.area_code = @area_code";
                    lstParam.Add(new SqlParameter("@area_code", areaCode));
                }

                //ログインユーザがシステム権限を保持している かつ 部門が選択されている場合
                if (user.Roles.Contains(ComConst.SYSTEM_OPERATE_USER) && !string.IsNullOrEmpty(ccDepartmentSelector.GetSelectedValue()))
                {
                    strSQL += " AND psn.selectionID = @selectionID";
                    lstParam.Add(new SqlParameter("@selectionID", ccDepartmentSelector.GetSelectedValue()));
                }

                DataTable result = new DataTable();

                try
                {
                    //SQL実行
                    m_ComFunc.DB_Select(strSQL, lstParam, ref result);
                }
                catch (Exception ex)
                {
                    logger.Error("User:" + user.UserId + ex.Message, ex);

                    lblErrMessage.Text = ex.Message;
                }

                //取得データ画面表示処理
                if (result.Rows.Count > 0)
                {
                    result.TableName = "resultTable";
                    result.Columns.Add("selectionDate", typeof(string));
                    result.Columns.Add("dismissDate", typeof(string));
                    result.Columns.Add("SB/WCP", typeof(string));
                    result.Columns.Add(areaName, typeof(string));

                    var lstSbwcp = m_ComFunc.GetSbWcpList();

                    foreach (DataRow row in result.Rows)
                    {
                        if (row.Field<DateTime?>("選任日") != null)
                        {
                            row.SetField<string>("selectionDate", row.Field<DateTime>("選任日").ToString("yyyy/MM/dd"));
                        }
                        if (row.Field<DateTime?>("解任日") != null)
                        {
                            row.SetField<string>("dismissDate", row.Field<DateTime>("解任日").ToString("yyyy/MM/dd"));
                        }
                        row.SetField<string>("SB/WCP", lstSbwcp[row.Field<int>("sb_wcp").ToString()]);

                        row.SetField<string>(areaName, row.Field<string>("thcAreaName"));
                    }

                    Session["RESULT"] = result;
                    displayGvApplicationTarget();
                    gvOutputStyleBumon.Visible = false;
                    gvApplicationTarget.Visible = true;
                    PnlApplicationTarget.Visible = true;
                }
                else
                {
                    Session["RESULT"] = null;
                    gvApplicationTarget.DataSource = result;
                    gvApplicationTarget.DataBind();
                    gvApplicationTarget.Visible = true;

                    lblErrMessage.Text = "検索条件に一致するデータが存在しませんでした。";
                }
            }
            else if (radioBumon.Checked) //一括出力が選択されている場合
            {
                radioRegional.Enabled = false;
                //一括出力用の検索処理を実行
                List<SqlParameter> lstParam = new List<SqlParameter>();

                //申請対象リスト検索処理
                string strSQL = string.Empty;
                strSQL += " SELECT";
                strSQL += "   * ";
                strSQL += " FROM";
                strSQL += "   ( ";
                strSQL += "SELECT";
                strSQL += " RANK() OVER ( PARTITION BY psn.enginnerID, psn.selectionID ORDER BY request_date DESC ) AS rank ";
                strSQL += " , ROW_NUMBER() OVER (ORDER BY psn.enginnerID,psn.selectionID) AS 'No'";
                strSQL += " , eng.LastName";
                strSQL += " , eng.FirstName";
                strSQL += " , eng.LastName_Kana";
                strSQL += " , eng.FirstName_Kana";
                strSQL += " , eng.getlicense_date";
                strSQL += " , eng.license_type";
                strSQL += " , eng.license_number";
                strSQL += " , '' AS '業務履歴'";
                strSQL += " , '' AS '固定値'";
                strSQL += " , psn.belong_company";
                strSQL += " , psn.belong_department";
                strSQL += " , psn.jmcia_flag ";
                strSQL += " , psn.sb_wcp ";

                //不要
                //strSQL += " ,date1.sr_date AS '選任日'";
                //strSQL += " ,date2.sr_date AS '解任日'";

                //提出先総通局が選択されていない場合は全地域を出力
                if (string.IsNullOrEmpty(strSotsuKyoku))
                {
                    strSQL += " , psn.hokkaidou ";
                    strSQL += " , psn.touhoku ";
                    strSQL += " , psn.hokuriku ";
                    strSQL += " , psn.shinetsu ";
                    strSQL += " , psn.kantou ";
                    strSQL += " , psn.tokai ";
                    strSQL += " , psn.kinki ";
                    strSQL += " , psn.shikoku ";
                    strSQL += " , psn.chugoku ";
                    strSQL += " , psn.kyuushu ";
                    strSQL += " , psn.okinawa ";
                }
                else
                {
                    foreach (string name in areaRowNames)
                    {
                        if (name.Equals(m_ComFunc.GetAreaRowName(strSotsuKyoku)))
                        {
                            strSQL += " , psn." + name + " ";
                        }
                        else
                        {
                            strSQL += " , '' as " + name + " ";
                        }
                    }
                }


                strSQL += " FROM selection_personal_tbl psn";
                strSQL += " INNER JOIN radio_enginner_info_tbl eng";
                strSQL += " ON eng.enginnerID = psn.enginnerID";

                //選任解任日は出力しないため不要な想定
                //strSQL += " LEFT OUTER JOIN selection_date_tbl date1";
                //strSQL += " ON date1.enginnerID = psn.enginnerID";
                //strSQL += " AND date1.selectionID = psn.selectionID";
                //strSQL += " AND date1.request_date = psn.request_date";
                //strSQL += " AND date1.sr_flag = 1";
                //strSQL += " LEFT OUTER JOIN selection_date_tbl date2";
                //strSQL += " ON date2.enginnerID = psn.enginnerID";
                //strSQL += " AND date2.selectionID = psn.selectionID";
                //strSQL += " AND date2.request_date = psn.request_date";
                //strSQL += " AND date2.sr_flag = 2";
                //strSQL += " AND date2.area_code = date1.area_code";

                //Where句
                strSQL += " WHERE psn.delete_flag = 0";

                // 承認済みのデータのみを表示
                strSQL += " AND psn.apploval_flag = 2";

                //所属会社
                if (!string.IsNullOrEmpty(strCompany))
                {
                    strSQL += " AND psn.belong_company like @company";
                    //パラメータ設定
                    lstParam.Add(new SqlParameter("@company", "%" + strCompany + "%"));
                }

                //所属部署
                if (!string.IsNullOrEmpty(strDepartment))
                {
                    strSQL += " AND psn.belong_department like @department";
                    lstParam.Add(new SqlParameter("@department", "%" + strDepartment + "%"));
                }

                //JMCIAフラグ
                if (jmciaFlag != null)
                {
                    strSQL += " AND psn.jmcia_flag = @jmcia";
                    lstParam.Add(new SqlParameter("@jmcia", jmciaFlag));
                }

                //SBWCPフラグ
                if (sbwcpFlag != null)
                {
                    strSQL += " AND psn.sb_wcp = @sbwcp";
                    lstParam.Add(new SqlParameter("@sbwcp", sbwcpFlag));
                }

                ////提出先総通局
                //if (!string.IsNullOrEmpty(strSotsuKyoku))
                //{
                //    int areaCode = m_ComFunc.getAreaCode(strSotsuKyoku);
                //    string rowName = m_ComFunc.GetAreaRowName(strSotsuKyoku);

                //    strSQL += " AND psn." + rowName + " IS NOT NULL AND LEN(psn." + rowName + ") > 0";
                //    if (chkEX.Checked) strSQL += " AND psn." + rowName + " like '%EX%'";
                //    if (chkFX.Checked) strSQL += " AND psn." + rowName + " like '%FX%'";
                //    if (chkFB.Checked) strSQL += " AND psn." + rowName + " like '%FB%'";
                //    if (chkFBR.Checked) strSQL += " AND psn." + rowName + " like '%FBR%'";
                //    if (chkML.Checked) strSQL += " AND psn." + rowName + " like '%ML%'";

                //    strSQL += " AND date1.area_code = @area_code";
                //    lstParam.Add(new SqlParameter("@area_code", areaCode));
                //}

                //部門が選択されている場合
                if (!string.IsNullOrEmpty(ccDepartmentSelector.GetSelectedValue()))
                {
                    strSQL += " AND psn.selectionID = @selectionID";
                    lstParam.Add(new SqlParameter("@selectionID", ccDepartmentSelector.GetSelectedValue()));
                }

                strSQL += "   ) A ";
                strSQL += " WHERE";
                strSQL += "   A.rank = 1";

                DataTable result = new DataTable();

                try
                {
                    //SQL実行
                    m_ComFunc.DB_Select(strSQL, lstParam, ref result);
                }
                catch (Exception ex)
                {
                    logger.Error("User:" + user.UserId + ex.Message, ex);

                    lblErrMessage.Text = ex.Message;
                }

                //取得データ画面表示処理
                if (result.Rows.Count > 0)
                {
                    result.Columns.Remove("rank");
                    Session["RESULT"] = result;
                    DataTable displayDt = result.Copy();
                    displayDt.Columns.Add("displayGetLicenseDate",typeof(string));
                    foreach (DataRow row in displayDt.Rows)
                    {
                        row.SetField<string>("displayGetLicenseDate", row.Field<DateTime>("getlicense_date").ToString("yyyy/MM/dd"));
                    }

                    PnlApplicationTarget.Visible = true;
                    gvApplicationTarget.Visible = false;
                    gvOutputStyleBumon.Visible = true;
                    //gvOutputStyleBumon.DataSource = result;
                    gvOutputStyleBumon.DataSource = displayDt;
                    gvOutputStyleBumon.DataBind();
                }
                else
                {
                    Session["RESULT"] = null;
                    gvApplicationTarget.Visible = false;
                    gvOutputStyleBumon.DataSource = result;
                    gvOutputStyleBumon.DataBind();
                    gvOutputStyleBumon.Visible = true;
                    lblErrMessage.Text = "検索条件に一致するデータが存在しませんでした。";
                }

            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + user.UserId + "検索ボタン押下処理");
        }

        //クリアボタン押下時処理
        protected void BtnClear_Click(object sender, EventArgs e)
        {
            UserRole userRole=    Session[ComConst.SESSION_USER_ROLE] as UserRole;
            Session["RESULT"] = null;

            txtSheetName.Text = string.Empty;
            if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
            {
                ccDepartmentSelector.InitializeControls();
                ddlSotsu.SelectedIndex = 0;
            }
            ddlCompany.SelectedIndex = 0;
            ddlCompany_Changed(sender, e);
            ddlJmcia.SelectedIndex = 0;
            ddlSbwcp.SelectedIndex = 0;
            chkEX.Checked = false;
            chkFX.Checked=false;
            chkFB.Checked = false;
            chkFBR.Checked = false;
            chkML.Checked = false;

            radioRegional.Enabled = true;
            radioBumon.Enabled = true;
            PnlApplicationTarget.Visible = false;
        }

        protected void modoru_Click(object sender, EventArgs e)
        {
            Response.Redirect("Mainmenu.aspx", false);
        }

        /// <summary>
        /// 出力ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void BtnDownload_Click(object sender, EventArgs e)
        {
            UserRole user = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + user.UserId + "出力ボタン押下処理");

            //出力対象データ取得
            DataTable dt = Session["RESULT"] as DataTable;
            DataTable targetData = CreateOutputDataTable(dt.Columns);
            

            //地域ごとが選択されている場合
            if (radioRegional.Checked)
            {
                //出力用データテーブル作成処理
                //備考列を追加
                if (!dt.Columns.Contains("備考"))
                {
                    dt.Columns.Add(new DataColumn("備考", typeof(string)));
                }

                //備考データを画面から取得
                foreach (GridViewRow gvRow in gvApplicationTarget.Rows)
                {
                    int dtiIndex = gvRow.DataItemIndex;
                    System.Diagnostics.Debug.Print(gvRow.DataItemIndex.ToString());
                    dt.Rows[dtiIndex].SetField<string>("備考", (gvRow.Cells[9].FindControl("remarks") as TextBox).Text);
                }


                DataTable dtOutput = dt.Copy();
                dtOutput.Columns.Remove("selectionDate");
                dtOutput.Columns.Remove("dismissDate");
                dtOutput.Columns.Remove("sb_wcp");

                int rownum = 1;
                //GridViewからチェック行を取得する。
                foreach (GridViewRow gvRow in gvApplicationTarget.Rows)
                {
                    if ((gvRow.Cells[0].FindControl("selectData") as CheckBox).Checked == true)
                    {
                        DataRow newRow = targetData.NewRow();

                        DataRow row = dtOutput.Rows[gvRow.DataItemIndex];
                        foreach (DataColumn dc in targetData.Columns)
                        {
                            if (dc.ColumnName.Equals("No"))
                            {
                                newRow.SetField<string>(dc.ColumnName, rownum.ToString());
                            }
                            else
                            {
                                newRow.SetField<string>(dc.ColumnName, row[dc.ColumnName].ToString());
                            }
                        }
                        targetData.Rows.Add(newRow);
                        rownum++;
                    }
                }
            }
            else if (radioBumon.Checked)
            {
                targetData = dt.Clone();
                DataTable dtOutput = dt.Copy();
                int rownum = 1;

                //GridViewからチェック行を取得する。
                foreach (GridViewRow gvRow in gvOutputStyleBumon.Rows)
                {
                    if ((gvRow.Cells[0].FindControl("selectData") as CheckBox).Checked == true)
                    {
                        DataRow newRow = targetData.NewRow();

                        DataRow row = dtOutput.Rows[gvRow.DataItemIndex];
                        foreach (DataColumn dc in targetData.Columns)
                        {
                            if (dc.ColumnName.Equals("No"))
                            {
                                newRow.SetField<string>(dc.ColumnName, rownum.ToString());
                            }
                            else
                            {
                                newRow.SetField<string>(dc.ColumnName, row[dc.ColumnName].ToString());
                            }
                        }
                        targetData.Rows.Add(newRow);
                        rownum++;
                    }
                }
            }
            //ダウンロードされるファイル名
            string downloadFileName = string.Empty;
            string defaultFileName = ConfigurationManager.AppSettings[ComConst.CONFIG_APPLICATION_LIST_FILE_NAME];

            downloadFileName = m_ComFunc.GetDownloadFileName(txtDownloadFileName.Text,defaultFileName);

            //出力ファイル生成
            string folderName = Server.MapPath("~/donwload/");
            
            string fullPath = string.Empty;

            //サーバに作成されるファイル名
            string serverFileName = m_ComFunc.CreateServerFileName(Session.SessionID);

            //作成されるファイルのフルパス
            fullPath = Path.Combine(folderName, serverFileName);

            //Excelファイルを作成
            try
            {
                if (radioBumon.Checked)
                {
                    m_ComFunc.CreateExcelFileByFormat(targetData, fullPath, strSheetName);
                }
                else if (radioRegional.Checked)
                {
                    m_ComFunc.CreateExcelFile(targetData, fullPath, strSheetName);
                }
            }
            catch(Exception ex)
            {
                lblErrMessage.Text = "Excelファイルの作成に失敗しました。";
                logger.Error("User:" + user.UserId + ex.Message, ex);
            }

            //ダウンロード処理を実行
            try
            {
                m_ComFunc.DownloadExcelFile(this.Response, downloadFileName, fullPath);
            }
            catch(Exception ex)
            {
                lblErrMessage.Text = "ファイルのダウンロードに失敗しました。";
                logger.Error("User:" + user.UserId + ex.Message, ex);
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + user.UserId + "出力ボタン押下処理");
        }

        /// <summary>
        /// GridViewページャ押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvPageChanging(object sender, GridViewPageEventArgs e)
        {
            gvApplicationTarget.PageIndex = e.NewPageIndex;
            gvApplicationTarget.PageSize = gvPagesize;
            gvApplicationTarget.DataSource = Session["RESULT"] as DataTable;
            gvApplicationTarget.DataBind();
        }

        protected void gvOutputStyleBumon_PageChanging(object sender, GridViewPageEventArgs e)
        {
            gvOutputStyleBumon.PageIndex = e.NewPageIndex;
            gvOutputStyleBumon.PageSize = gvPagesize;
            gvOutputStyleBumon.DataSource = Session["RESULT"] as DataTable;
            gvOutputStyleBumon.DataBind();
        }

        protected void radioBumon_CheckedChanged(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //システム管理者の場合
            if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
            {
                ddlSotsu.Items.Clear();
                if (radioBumon.Checked)
                {
                    createDdlSotsukyokuList(true);
                }
            }
        }

        protected void radioRegional_CheckedChanged(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //システム管理者の場合
            if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
            {
                ddlSotsu.Items.Clear();

                if (radioRegional.Checked)
                {
                    createDdlSotsukyokuList(false);
                }
            }
        }


        #endregion イベント定義

        #region PrivateUtility
        private void displayGvApplicationTarget()
        {
            gvApplicationTarget.DataSource = Session["RESULT"] as DataTable;
            gvApplicationTarget.PageSize = gvPagesize;
            gvApplicationTarget.DataBind();
        }
        private DataTable CreateOutputDataTable(DataColumnCollection columns)
        {
            DataTable dt = new DataTable();
            dt.Columns.Add(new DataColumn("No", typeof(string)));
            dt.Columns.Add(new DataColumn("氏名（姓）", typeof(string)));
            dt.Columns.Add(new DataColumn("氏名（名）", typeof(string)));
            dt.Columns.Add(new DataColumn("ふりがな（姓）", typeof(string)));
            dt.Columns.Add(new DataColumn("ふりがな（名）", typeof(string)));
            dt.Columns.Add(new DataColumn("免許番号", typeof(string)));
            dt.Columns.Add(new DataColumn("JMCIA", typeof(string)));
            dt.Columns.Add(new DataColumn("SB/WCP", typeof(string)));
            dt.Columns.Add(new DataColumn("所属会社", typeof(string)));
            dt.Columns.Add(new DataColumn("選任日", typeof(string)));
            dt.Columns.Add(new DataColumn("解任日", typeof(string)));
            dt.Columns.Add(new DataColumn(m_ComFunc.GetAreaName(strSotsuKyoku), typeof(string)));
            dt.Columns.Add(new DataColumn("備考", typeof(string)));

            return dt;
        }

        private void createDdlSotsukyokuList(bool hasAllArea)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //総通局DDL生成
            DataTable dtSotsuKyokuList = m_ComFunc.GetSotsuKyokuList();

            //本部ユーザ以外の場合、表示対象の地域を絞り込む
            if (!userRole.Areas.Contains("HQ"))
            {
                dtSotsuKyokuList = dtSotsuKyokuList.AsEnumerable().Where(row => userRole.Areas.Contains(row.Field<string>("code"))).CopyToDataTable();
            }
            else
            {
                //本部ユーザで全国が必要な場合は全国を追加する。
                if (hasAllArea)
                {
                    
                    ddlSotsu.Items.Add(new ListItem("全国", string.Empty));
                }
            }

            foreach (DataRow row in dtSotsuKyokuList.Rows)
            {
                ddlSotsu.Items.Add(new ListItem(row.Field<string>("code_name"), row.Field<string>("code")));
            }
        }

        #endregion PrivateUtility

    }
}