﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Web.UI.WebControls;
using EmpCRM.common;
using EmpCRM.Dto;

namespace EmpCRM
{
    /// <summary>
    /// 申請状況確認画面
    /// </summary>
    public partial class ApplicationStatusList : System.Web.UI.Page
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        //共通クラス
        private ComFunc m_ComFunc = new ComFunc();
        //GridViewページサイズ
        private static int gvPagesize = int.Parse(ConfigurationManager.AppSettings[ComConst.CONFIG_GRIDVIEW_PAGESIZE]);

        #region 画面項目用プロパティ定義
        //ステータス
        private int? requestStatus = null;
        //免許証番号
        private string strLicenseNo = string.Empty;
        //資格
        private string strLicenseType = string.Empty;
        //社内管理部門
        private string strBumon = string.Empty;
        //氏名（姓）
        private string strLastName = string.Empty;
        //氏名（名）
        private string strFirstName = string.Empty;
        //総合通信局
        private string strSotsuKyoku = string.Empty;
        //ふりがな（姓）
        private string strKanaLastName = string.Empty;
        //ふりがな（名）
        private string strKanaFirstName = string.Empty;
        //所属会社
        private string strCompany = string.Empty;
        //所属部署
        private string strDepartment = string.Empty;
        //JMCIA
        private int? jmcia = null;
        //SB/WCP
        private int? sbwcp = null;

        //申請状況一覧
        private static DataTable dtApplicationStatusList = null;
        #endregion 画面項目用プロパティ定義

        #region イベント定義
        protected void Page_Load(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //ログイン状態チェック
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                lblErrorMessage.Text = string.Empty;

                if (!IsPostBack)
                {
                    try
                    {
                        //ステータス
                        //申請ステータスリストを取得（生成）
                        var lstAppFlag = m_ComFunc.GetApprovalFlagList();
                        ddlStatusList.Items.Add(new ListItem(string.Empty, string.Empty));
                        foreach (KeyValuePair<string, string> pair in lstAppFlag)
                        {
                            ddlStatusList.Items.Add(new ListItem(pair.Value, pair.Key));
                        }
                    }
                    catch(Exception ex)
                    {
                        lblErrorMessage.Text += " ステータス取得失敗 ";
                        logger.Error("ステータス取得失敗", ex);
                    }

                    try
                    {
                        //資格名の一覧を取得
                        var dictLicenseTypeName = m_ComFunc.GetLicenseNameList();

                        //資格一覧をセッションに設定
                        Session[ComConst.SESSION_LICENSE_NAME_MASTER] = dictLicenseTypeName;

                        //資格DDLを設定
                        DataTable dtLicenseList = new DataTable();

                        foreach (KeyValuePair<string, string> pair in dictLicenseTypeName)
                        {
                            ddlLicenseTypeList.Items.Add(new ListItem(pair.Value,pair.Key));
                        }
                    }
                    catch (Exception ex)
                    {
                        lblErrorMessage.Text += " 資格取得失敗 ";
                        logger.Error("資格取得失敗", ex);
                    }

                    try
                    {
                        //社内管理部門
                        //システム管理者の場合
                        if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
                        {
                            ccDepartmentSelector.InitializeControls();
                            ccDepartmentSelector.Visible = true;
                        }
                        else //システム管理者以外の場合
                        {
                            //主任部門のみを表示
                            lblDepartmentName.Text = userRole.DepartmentName;
                            lblDepartmentName.Visible = true;
                        }
                    }
                    catch (Exception ex)
                    {
                        lblErrorMessage.Text += " 社内管理部門取得失敗 ";
                        logger.Error("社内管理部門取得失敗", ex);
                    }

                    try
                    {
                        //総合通信局
                        DataTable dtSotsuKyokuList = new DataTable();
                        if (m_ComFunc.GetSotsuKyokuList(ref dtSotsuKyokuList))
                        {
                            //本部以外のユーザの場合は地域を絞り込む
                            if (!userRole.Areas.Contains("HQ"))
                            {
                                dtSotsuKyokuList = dtSotsuKyokuList.AsEnumerable().Where(row => userRole.Areas.Contains(row.Field<string>("code"))).CopyToDataTable();
                            }

                            ddlSotsuKyokuList.Items.Add(new ListItem(string.Empty, string.Empty));
                            foreach (DataRow row in dtSotsuKyokuList.Rows)
                            {
                                ddlSotsuKyokuList.Items.Add(new ListItem(row.Field<string>("code_name"), row.Field<string>("code")));
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        lblErrorMessage.Text += " 総合通信局取得失敗 ";
                        logger.Error("総合通信局取得失敗", ex);
                    }

                    try
                    {
                        //所属会社DDL
                        m_ComFunc.SetBelongCompanyDropdownlist(ddlCompanyList);
                        //ddlCompanyList.Items.Add(new ListItem(string.Empty, string.Empty));
                        //DataTable dtCompanyList = m_ComFunc.GetCompanyMaster();
                        //if (dtCompanyList != null && dtCompanyList.Rows.Count > 0)
                        //{
                        //    foreach (DataRow row in dtCompanyList.Rows)
                        //    {
                        //        ddlCompanyList.Items.Add(new ListItem(row.Field<string>("company_name"), row.Field<string>("company_id")));
                        //    }
                        //}
                    }
                    catch(Exception ex)
                    {
                        lblErrorMessage.Text += " 所属会社取得失敗 ";
                        logger.Error("所属会社取得失敗", ex);
                    }

                    //所属部署
                    ddlDepartmentList.Items.Add(new ListItem(string.Empty, string.Empty));

                    //JMCIA
                    ddlJmcia.Items.Add(new ListItem(string.Empty, string.Empty));
                    ddlJmcia.Items.Add(new ListItem("OFF", "0"));
                    ddlJmcia.Items.Add(new ListItem("ON", "1"));


                    //SB/WCP
                    //マスタからSB/WCPの値を取得
                    var lstSbwcp = m_ComFunc.GetSbWcpList();
                    ddlSbwcp.Items.Add(new ListItem(string.Empty, string.Empty));
                    foreach (KeyValuePair<string,string> pair in lstSbwcp)
                    {
                        ddlSbwcp.Items.Add(new ListItem(pair.Value, pair.Key));

                    }
                }
                else
                {
                }
            }
        }

        /// <summary>
        /// 選択中の所属会社変更時に所属部署一覧を再取得する。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddlCompanyList_Changed(object sender, EventArgs e)
        {
            //選択された会社に応じて部署を設定7
            m_ComFunc.SetBelongDepartmentDropdownlist(ddlDepartmentList, ddlCompanyList.SelectedValue.ToString());

            if (dtApplicationStatusList != null)
            {
                DisplayApplicationList();
            }
        }

        //検索ボタン押下時
        protected void btnSearch_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "検索ボタン押下時");

            //画面入力値を取得

            //ステータス
            if (!string.IsNullOrEmpty(ddlStatusList.SelectedValue))
            {
                requestStatus = int.Parse(ddlStatusList.SelectedValue);
            }
            //免許証番号
            strLicenseNo = txtLicenseNo.Text;
            //資格種別
            strLicenseType = ddlLicenseTypeList.SelectedValue;
            //社内管理部門
            //システムユーザの場合ドロップダウンリストから取得
            if (userRole.Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
            {
                strBumon = ccDepartmentSelector.GetSelectedValue();
            }
            else
            {
                strBumon = userRole.DepartmentId;
            }
            //氏名（姓）
            strLastName = txtLastName.Text;
            //氏名（名）
            strFirstName = txtFirstName.Text;
            //総合通信局
            strSotsuKyoku = ddlSotsuKyokuList.SelectedValue;
            //ふりがな（姓）
            strKanaLastName = txtKanaLastName.Text;
            //ふりがな（名）
            strKanaFirstName = txtKanaFirstName.Text;
            //所属会社
            strCompany = ddlCompanyList.SelectedItem.Text;
            //所属部署
            strDepartment = ddlDepartmentList.SelectedItem.Text;
            //JMCIA
            if (string.IsNullOrEmpty(ddlJmcia.SelectedValue.ToString()))
            {
                jmcia = null;
            }
            else
            {
                jmcia = int.Parse(ddlJmcia.SelectedValue.ToString());
            }
            //SB/WCP
            if (string.IsNullOrEmpty(ddlSbwcp.SelectedValue.ToString()))
            {
                sbwcp = null;
            }
            else
            {
                sbwcp = int.Parse(ddlSbwcp.SelectedValue.ToString());
            }

            List<SqlParameter> lstParam = new List<SqlParameter>();

            //入力された条件で検索
            string strSQL = string.Empty;
            strSQL += "SELECT";
            strSQL += " ROW_NUMBER() OVER(order by psn.request_date ) 'No', ";
            strSQL += " CONVERT(varchar,psn.apploval_flag) AS 'ステータス',";
            strSQL += " psn.selectionID AS '社内管理部門',";
            strSQL += " eng.LastName AS '氏名（姓）',";
            strSQL += " eng.FirstName AS '氏名（名）',";
            strSQL += " eng.LastName_Kana AS 'ふりがな（姓）',";
            strSQL += " eng.FirstName_Kana AS 'ふりがな（名）',";
            strSQL += " psn.belong_company AS '所属会社',";
            strSQL += " psn.belong_department AS '所属部署',";
            strSQL += " CASE psn.jmcia_flag WHEN '0' THEN '' WHEN '1' THEN '●' END AS 'JMCIA',";
            strSQL += " str(psn.sb_wcp) 'SB/WCP' ,";
            strSQL += " eng.license_number AS '免許番号',";
            strSQL += " mst2.code_name AS '資格名'";
            strSQL += " FROM selection_personal_tbl psn";
            strSQL += " INNER JOIN radio_enginner_info_tbl eng ON eng.enginnerID = psn.enginnerID";
            strSQL += " LEFT JOIN code_mst mst2 ON mst2.code_type = '01' AND mst2.code = eng.license_type";
            strSQL += " where delete_flag = 0";
            //ステータス
            if (requestStatus != null)
            {
                strSQL += " and psn.apploval_flag = @apploval_flag";
                lstParam.Add(new SqlParameter("@apploval_flag", requestStatus));
            }
            //免許証番号
            if (!string.IsNullOrEmpty(strLicenseNo))
            {
                strSQL += " and eng.license_number LIKE @license_number";
                lstParam.Add(new SqlParameter("@license_number", "%" + strLicenseNo + "%"));
            }
            //資格
            if (!string.IsNullOrEmpty(strLicenseType))
            {
                strSQL += " and eng.license_type = @license_type";
                lstParam.Add(new SqlParameter("@license_type", strLicenseType));
            }
            //社内管理部門
            if (!string.IsNullOrEmpty(strBumon))
            {
                strSQL += " and psn.selectionID = @selectionID";
                lstParam.Add(new SqlParameter("@selectionID", strBumon));
            }
            //氏名（姓）
            if (!string.IsNullOrEmpty(strLastName))
            {
                strSQL += " and eng.LastName LIKE @LastName";
                lstParam.Add(new SqlParameter("@LastName", "%" + strLastName + "%"));
            }
            //氏名（名）
            if (!string.IsNullOrEmpty(strFirstName))
            {
                strSQL += " and eng.FirstName LIKE @FirstName";
                lstParam.Add(new SqlParameter("@FirstName", "%" + strFirstName + "%"));
            }
            //ふりがな（姓）
            if (!string.IsNullOrEmpty(strKanaLastName))
            {
                strSQL += " and eng.LastName_Kana LIKE @LastName_Kana";
                lstParam.Add(new SqlParameter("@LastName_Kana", "%" + strKanaLastName + "%"));
            }
            //ふりがな（名）
            if (!string.IsNullOrEmpty(strKanaFirstName))
            {
                strSQL += " and eng.FirstName_Kana LIKE @FirstName_Kana";
                lstParam.Add(new SqlParameter("@FirstName_Kana", "%" + strKanaFirstName + "%"));
            }
            //所属会社
            if (!string.IsNullOrEmpty(strCompany))
            {
                strSQL += " and psn.belong_company LIKE @belong_company";
                lstParam.Add(new SqlParameter("@belong_company", "%" + strCompany + "%"));
            }
            //所属部署
            if (!string.IsNullOrEmpty(strDepartment))
            {
                strSQL += " and psn.belong_department LIKE @belong_department";
                lstParam.Add(new SqlParameter("@belong_department", "%" + strDepartment + "%"));
            }
            //JMCIA
            if (jmcia != null)
            {
                strSQL += " and psn.jmcia_flag = @jmcia_flag";
                lstParam.Add(new SqlParameter("@jmcia_flag", jmcia));
            }
            //SB/WCP
            if (sbwcp != null)
            {
                strSQL += " and psn.sb_wcp = @sb_wcp";
                lstParam.Add(new SqlParameter("@sb_wcp", sbwcp));
            }
            //総合通信局
            if (!string.IsNullOrEmpty(strSotsuKyoku))
            {
                strSQL += " AND (psn." + m_ComFunc.GetAreaRowName(strSotsuKyoku) + " IS NOT NULL AND DATALENGTH(psn." + m_ComFunc.GetAreaRowName(strSotsuKyoku) + ") <> 0)";
            }

            DataTable result = new DataTable();

            try
            {
                if (m_ComFunc.DB_Select(strSQL, lstParam, ref result))
                {
                    //SQL正常終了
                    //検索結果を設定
                    dtApplicationStatusList = result;

                    //セッションから部門一覧を取得
                    var bumonDict = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string, string>;

                    //社内管理部門列の長さを200に設定しておく。
                    dtApplicationStatusList.Columns["社内管理部門"].MaxLength = 200;

                    //マスタからSBWCPの一覧を取得
                    var lstSbwcp = m_ComFunc.GetSbWcpList();
                    dtApplicationStatusList.Columns["SB/WCP"].ReadOnly = false;

                    var appFlagList= m_ComFunc.GetApprovalFlagList();

                    //セッションから資格一覧を取得
                    var dictLicenseTypeName = Session[ComConst.SESSION_LICENSE_NAME_MASTER] as Dictionary<string, string>;

                    foreach (DataRow row in dtApplicationStatusList.Rows)
                    {
                        dtApplicationStatusList.Columns["ステータス"].ReadOnly = false;
                        // 申請ステータス値を名称に置換
                        row.SetField<string>("ステータス", appFlagList[row.Field<string>("ステータス")]);

                        //部門ID⇒部門名に置換
                        row.SetField<string>("社内管理部門", m_ComFunc.GetSelectionSectionName(row.Field<string>("社内管理部門"), bumonDict));

                        //SB/WCPの値を名称に置換
                        var sbwcpKey = row.Field<string>("SB/WCP").Trim();
                        if (lstSbwcp.ContainsKey(sbwcpKey))
                        {
                            row.SetField<string>("SB/WCP", lstSbwcp[sbwcpKey]);
                        }
                        else
                        {
                            //不正なキーの場合はそのまま表示
                            row.SetField<string>("SB/WCP", sbwcpKey);
                        }

                        //資格番号から資格名を取得
                        row.SetField<string>("資格名", dictLicenseTypeName[m_ComFunc.GetLicenseTypeCode(row.Field<string>("免許番号"))]);
                        
                    }
                }
                else
                {
                    lblErrorMessage.Text = "データの取得に失敗しました。";
                    pnlDisplayErrorArea.Visible = true;
                    return;
                }
            }
            catch (Exception ex)
            {
                lblErrorMessage.Text += "データの取得に失敗しました。";
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
            }

            if (dtApplicationStatusList != null && dtApplicationStatusList.Rows.Count > 0)
            {
                //検索結果を画面に設定
                DisplayApplicationList();

                //画面描画
                pnlApplicationStatusList.Visible = true;
            }
            else
            {
                //pnlApplicationStatusList.Visible = false;
                DisplayApplicationList();
                pnlApplicationStatusList.Visible = true;

                lblErrorMessage.Text = "検索条件に一致するデータが存在しません。";
                pnlDisplayErrorArea.Visible = true;
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "検索ボタン押下時");
        }



        //クリアボタン押下時処理
        protected void btnClear_Click(object sender, EventArgs e)
        {
            //プロパティ値をクリア
            requestStatus = null;
            strLicenseNo = string.Empty;
            strLicenseType = string.Empty;
            strBumon = string.Empty;
            strLastName = string.Empty;
            strFirstName = string.Empty;
            strSotsuKyoku = string.Empty;
            strKanaLastName = string.Empty;
            strKanaFirstName = string.Empty;
            strCompany = string.Empty;
            strDepartment = string.Empty;
            jmcia = null;
            sbwcp = null;

            //画面入力項目を初期化
            ddlStatusList.SelectedIndex = 0;
            if ((Session[ComConst.SESSION_USER_ROLE] as UserRole).Roles.Contains(ComConst.SYSTEM_OPERATE_USER))
            {
                ccDepartmentSelector.InitializeControls();
            }
            ddlSotsuKyokuList.SelectedIndex = 0;
            ddlCompanyList.SelectedIndex = 0;
            ddlCompanyList_Changed(sender,e);
            ddlLicenseTypeList.SelectedIndex = 0;
            ddlJmcia.SelectedIndex = 0;
            ddlSbwcp.SelectedIndex = 0;
            txtLicenseNo.Text = string.Empty;
            txtLastName.Text = string.Empty;
            txtFirstName.Text = string.Empty;
            txtKanaLastName.Text = string.Empty;
            txtKanaFirstName.Text = string.Empty;
            txtDownloadFileName.Text = string.Empty;

            //検索結果表示欄を非表示にする。
            pnlApplicationStatusList.Visible = false;

            //検索結果データをクリアする。
            dtApplicationStatusList = null;
        }

        //戻るボタン押下時
        protected void modoru_Click(object sender, EventArgs e)
        {
            Response.Redirect("Mainmenu.aspx", false);
        }

        //出力ボタン押下時処理
        protected void btnDownloadList_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "出力ボタン押下時");

            //ダウンロードファイル名を設定
            string strDownloadFileName = string.Empty;
            strDownloadFileName = m_ComFunc.GetDownloadFileName(txtDownloadFileName.Text,ConfigurationManager.AppSettings[ComConst.CONFIG_APPLICATION_STATUS_LIST_FILE_NAME]);

            //出力データを取得する。
            DataTable dtAppStatusList = dtApplicationStatusList.Copy();

            //Excelファイルに出力する。
            string folderName = Server.MapPath("~/donwload/");

            //サーバで作成されるファイル名
            string serverFileName = m_ComFunc.CreateServerFileName(Session.SessionID);

            //作成されるファイルのフルパス
            string serverFullPath = Path.Combine(folderName, serverFileName);

            //Excelファイル作成処理
            try
            {
                m_ComFunc.CreateExcelFile(dtAppStatusList, serverFullPath, string.Empty);
            }
            catch (Exception ex)
            {
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
                lblErrorMessage.Text = "Excelファイルの作成に失敗しました。";
                pnlDisplayErrorArea.Visible = true;
            }

            //ダウンロード処理実行
            try
            {
                m_ComFunc.DownloadExcelFile(this.Response, strDownloadFileName, serverFullPath);
            }
            catch (Exception ex)
            {
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
                lblErrorMessage.Text = "ダウンロード処理に失敗しました。";
                pnlDisplayErrorArea.Visible = true;
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "出力ボタン押下時");
        }

        protected void gvAppStatusList_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            gvApplicationStatusList.PageIndex = e.NewPageIndex;

            gvApplicationStatusList.DataSource = dtApplicationStatusList;
            gvApplicationStatusList.PageSize = gvPagesize;

            gvApplicationStatusList.DataBind();
        }

        #endregion イベント定義

        #region Private Method

        #region 申請状況一覧表示処理
        private void DisplayApplicationList()
        {
            gvApplicationStatusList.DataSource = dtApplicationStatusList;
            gvApplicationStatusList.PageSize = gvPagesize;
            gvApplicationStatusList.DataBind();
        }
        #endregion 申請状況一覧表示処理

        #endregion Private Method
    }
}