﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using EmpCRM.common;
using EmpCRM.Dto;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace EmpCRM
{
    /// <summary>
    /// 部門毎選任解任一覧画面
    /// </summary>
    public partial class BumonSelectedMissi : System.Web.UI.Page
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private ComFunc m_ComFunc = null;
        private static int gvPagesize = int.Parse(ConfigurationManager.AppSettings[ComConst.CONFIG_GRIDVIEW_PAGESIZE]);
        private string enginnerID_key = string.Empty;
        private string slicenseno = string.Empty;
        private string slastname = string.Empty;
        private string sfirstname = string.Empty;
        private const string DATE_FORMAT = "yyyy/M/d";
        /// <summary> 従事者情報一覧 </summary>
        private const string SESSION_WORKER_SEARCH_RESULT = "WorkerSearchResult";
        /// <summary> 選択された従事者情報 </summary>
        private const string SESSION_SELECTED_WORKER_ROW = "SelectedWokerRow";
        /// <summary> 選任解任状況一覧 </summary>
        private const string SESSION_SELECTION_CONDITION_SEARCH_RESULT = "SelectionConditionSearchResult";
        /// <summary> 選択された選任解任状況 </summary>
        private const string SESSION_SELECTED_SELECTION_CONDITION_ROW = "SelectedSelectionConditionRow";
        /// <summary> 詳細履歴検索結果 </summary>
        private const string SESSION_DETAIL_HISTORY_SEARCH_RESULT = "DetailHistorySearchResult";

        private const string SESSION_DETAIL_SELECTION_INFO = "DETAIL_SELECTION_INFO";

        private const string INFO_LOG_SEARCHBUTTON_PUSH = "検索ボタン押下処理";
        private const string INFO_LOG_BUMOSTATUSBUTTON_PUSH = "部門毎選任/解任状況ボタン押下処理";
        private const string INFO_LOG_SELECTIONDETAILBUTTON_PUSH = "詳細表示ボタン押下処理";
        //private const string INFO_LOG_SEARCHBTN_PUSH = "検索ボタン押下処理";

        /// <summary>
        /// 初期表示処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            //ログイン状態チェック
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                //共通処理クラスのインスタンス取得
                m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

                //初期表示時処理
                if (!IsPostBack)
                {
                    Session[SESSION_WORKER_SEARCH_RESULT] = null;
                    Session[SESSION_SELECTED_WORKER_ROW] = null;

                    //資格種別一覧を取得しておく
                    Session[ComConst.SESSION_LICENSE_NAME_MASTER] = m_ComFunc.GetLicenseNameList();

                    //所属会社DDL設定
                    m_ComFunc.SetBelongCompanyDropdownlist(ddlBelongCompany);
                }
            }
        }

        //所属会社選択時処理
        protected void ddlBelongCompany_Changed(object sender, EventArgs e)
        {
            m_ComFunc.SetBelongDepartmentDropdownlist(ddlBelongDepartment, ddlBelongCompany.SelectedValue.ToString());
        }

        #region 検索ボタン押下時処理
        /// <summary>
        /// 検索ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void SearchBtn_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + INFO_LOG_SEARCHBUTTON_PUSH);

            string sSql = string.Empty;
            bool bRet = false;
            DataTable resultTable = new DataTable();

            //従事者情報
            List<radio_enginner_info> rd_enginner_infoL = new List<radio_enginner_info>();

            //セッション初期化
            Session[ComConst.rd_enginner_infoL] = null;

            //SQL文作成
            sSql += "SELECT DISTINCT ";
            sSql += "   license_number";
            sSql += "   , LastName";
            sSql += "   , firstName";
            sSql += "   , enginnerID";
            sSql += "   , LastName_Kana";
            sSql += "   , FirstName_Kana";
            sSql += " FROM";
            sSql += "   enginner_info_search_view ";

            //条件設定する。
            string strWhere = string.Empty;


            //入力済み項目のSQLパラメータをリストに設定
            List<SqlParameter> lstSqlParameters = new List<SqlParameter>();

            //免許番号
            if (!string.IsNullOrEmpty(licenseno.Text))
            {
                strWhere += " license_number LIKE @license_number  ";
                lstSqlParameters.Add(new SqlParameter("@license_number", "%" + licenseno.Text + "%"));
            }

            //氏名(姓)
            if (!string.IsNullOrEmpty(lastname.Text))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " LastName LIKE @LastName  ";
                lstSqlParameters.Add(new SqlParameter("@LastName", "%" + lastname.Text + "%"));
            }

            //氏名(名)
            if (!string.IsNullOrEmpty(firstname.Text))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " firstName LIKE @firstName  ";
                lstSqlParameters.Add(new SqlParameter("@firstName", "%" + firstname.Text + "%"));
            }

            //ふりがな(姓)
            if (!string.IsNullOrEmpty(furi_familyname.Text))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " LastName_Kana LIKE @LastName_Kana  ";
                lstSqlParameters.Add(new SqlParameter("@LastName_Kana", "%" + furi_familyname.Text + "%"));
            }

            //ふりがな(名)
            if (!string.IsNullOrEmpty(furi_firstname.Text))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " FirstName_Kana LIKE @FirstName_Kana ";
                lstSqlParameters.Add(new SqlParameter("@FirstName_Kana", "%" + furi_firstname.Text + "%"));
            }

            //所属会社
            if (!string.IsNullOrEmpty(ddlBelongCompany.SelectedValue))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " belong_company = @belong_company";
                lstSqlParameters.Add(new SqlParameter("@belong_company", ddlBelongCompany.SelectedItem.Text));
            }

            //所属部署
            if (!string.IsNullOrEmpty(ddlBelongDepartment.SelectedValue))
            {
                if (!string.IsNullOrEmpty(strWhere)) strWhere += " AND ";
                strWhere += " belong_department = @belong_department";
                lstSqlParameters.Add(new SqlParameter("@belong_department", ddlBelongDepartment.SelectedItem.Text));
            }

            if (string.IsNullOrEmpty(strWhere))
            {
                sSql = sSql + "  ";
            }
            else
            {
                //Where句設定
                sSql = sSql + " WHERE " + strWhere;
            }

            //SQL文発行
            try
            {
                bRet = m_ComFunc.DB_Select(sSql, lstSqlParameters, ref resultTable);
            }
            catch (Exception ex)
            {
                lblErrMsg.Text = "データの取得に失敗しました。";
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
            }

            if (bRet == false)
            {
                lblErrMsg.Text = "検索できませんでした。";
            }

            resultTable.Columns.Add(new DataColumn("licenseName", typeof(string)));

            var dictLicenseTypeName = Session[ComConst.SESSION_LICENSE_NAME_MASTER] as Dictionary<string, string>;

            foreach (DataRow row in resultTable.Rows)
            {
                row.SetField<string>("licenseName", m_ComFunc.GetLicenseTypeName(row.Field<string>("license_number"),dictLicenseTypeName));
            }

            //検索結果をセッションに格納
            Session[SESSION_WORKER_SEARCH_RESULT] = resultTable;

            //検索結果を画面表示
            gvWorkerSearchResult.DataSource = resultTable;
            gvWorkerSearchResult.Visible = true;
            gvWorkerSearchResult.PageSize = gvPagesize;
            gvWorkerSearchResult.DataBind();

            //取得したDataTableをリストに設定
            foreach (DataRow row in resultTable.AsEnumerable())
            {
                radio_enginner_info info = new radio_enginner_info();
                info.license_number = row.Field<string>("license_number");
                info.LastName = row.Field<string>("LastName");
                info.FirstName = row.Field<string>("FirstName");
                info.enginnerID = row.Field<string>("enginnerID");
                info.LastName_Kana = row.Field<string>("LastName_Kana");
                info.FirstName_Kana = row.Field<string>("FirstName_Kana");

                //作成した情報をリストに追加
                rd_enginner_infoL.Add(info);
            }

            if (resultTable.Rows.Count == 0)
            {
                lblErrMsg.Text = "条件に一致する従事者情報が存在しません。";
            }

            //セッションに従事者情報を設定
            Session[ComConst.rd_enginner_infoL] = rd_enginner_infoL;

            // 従事者情報一覧を表示
            PnlWorkerSearchResult.Visible = true;

            //部門毎選任解任状況一覧非表示
            PnlBumonSelectionCondition.Visible = false;

            //選任解任詳細非表示
            PnlSelectionDetail.Visible = false;

            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + INFO_LOG_SEARCHBUTTON_PUSH);
        }
        #endregion 検索ボタン押下時処理

        #region 部門毎選任/解任状況ボタン押下時処理
        /// <summary>
        /// 部門毎選任/解任状況ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void BumonSelectionCondition_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + INFO_LOG_BUMOSTATUSBUTTON_PUSH);
            
            DataTable workerSearchResult = Session[SESSION_WORKER_SEARCH_RESULT] as DataTable;

            // ラジオボタンがチェックされている行を取得
            foreach (GridViewRow row in gvWorkerSearchResult.Rows)
            {
                if (((RadioButton)row.Cells[0].FindControl("rdSelectWorker")).Checked)
                {
                    //セッションに選択行を設定
                    Session[SESSION_SELECTED_WORKER_ROW] = workerSearchResult.Rows[row.DataItemIndex];
                    break;
                }
            }

            if (Session[SESSION_SELECTED_WORKER_ROW] == null)
            {
                lblErrMsg.Text = "従事者情報が選択されていません。";
            }

            //選任解任状況検索処理
            try
            {
                SearchSelectionPersonal();
            }
            catch (Exception ex)
            {
                lblErrMsg.Text = "データの取得に失敗しました。";
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
            }
            PnlBumonSelectionCondition.Visible = true;
            PnlSelectionDetail.Visible = false;

            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + INFO_LOG_BUMOSTATUSBUTTON_PUSH);
        }
        #endregion

        #region 選任解任状況検索・表示処理
        /// <summary>
        /// 選任解任状況検索・表示処理
        /// </summary>
        private void SearchSelectionPersonal()
        {
            string strSql = string.Empty;
            bool bRet = false;
            DataTable resultTable = new DataTable();

            try
            {
                // エラーメッセージ初期化
                lblErrMsg.Text = string.Empty;

                //selection_personal_tblを検索する。
                //共通処理クラスのインスタンス取得
                //m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];
                
                //select句を生成
                string strSelect = string.Empty;
                strSelect += "SELECT";
                strSelect += " RANK() OVER(PARTITION BY enginnerID,selectionID ORDER BY request_date DESC) AS rank,";
                strSelect += " enginnerID,";
                strSelect += " selectionID,";
                strSelect += " request_date,";
                strSelect += " request_div,";
                strSelect += " belong_company,";
                strSelect += " belong_department,";
                strSelect += " jmcia_flag,";
                strSelect += " sb_wcp,";
                strSelect += " hokkaidou,";
                strSelect += " touhoku,";
                strSelect += " hokuriku,";
                strSelect += " shinetsu,";
                strSelect += " kantou,";
                strSelect += " tokai,";
                strSelect += " kinki,";
                strSelect += " shikoku,";
                strSelect += " chugoku,";
                strSelect += " kyuushu,";
                strSelect += " okinawa,";
                strSelect += " apploval_flag,";
                strSelect += " delete_flag,";
                strSelect += " create_date,";
                strSelect += " create_name,";
                strSelect += " update_date,";
                strSelect += " update_name ";
                strSelect += " FROM selection_personal_tbl ";

                //where句を作成
                string strWhere = string.Empty;

                // セッションからチェック行を取得
                DataRow selectedRow = Session[SESSION_SELECTED_WORKER_ROW] as DataRow;

                // チェックされている行が存在する場合
                if(selectedRow!=null)
                {
                    List<System.Data.SqlClient.SqlParameter> sqlParams = new List<System.Data.SqlClient.SqlParameter>();

                    int counter = 0;
                        string strParam = "@param" + counter;
                        strWhere += " enginnerID = " + strParam + " ";
                        sqlParams.Add(new System.Data.SqlClient.SqlParameter(strParam, selectedRow.Field<string>("enginnerID")));

                    //SQL生成
                    strSql = strSelect + "Where " + strWhere;
                    
                    //SQL実行
                    bRet = m_ComFunc.DB_Select(strSql, sqlParams,ref resultTable);

                    //SQL実行に失敗した場合
                    if (bRet == false)
                    {
                        lblErrMsg.Text = "検索できませんでした。";
                    }
                    else
                    {
                        //表示用列を追加
                        resultTable.Columns.Add(new DataColumn("ステータス", typeof(string)));
                        resultTable.Columns.Add(new DataColumn("社内管理部門", typeof(string)));

                        //セッションから部門一覧を取得
                        Dictionary<string, string> bumon = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string, string>;

                        //ステータスの一覧を取得
                        var lstApprovalFlag = m_ComFunc.GetApprovalFlagList();

                        foreach (DataRow row in resultTable.Rows)
                        {
                            //選任部門IDから部門名を取得し、表示用列に設定
                            row.SetField<string>("社内管理部門", m_ComFunc.GetSelectionSectionName(row.Field<string>("selectionID"), bumon));

                            //申請区分（コード）を名称に変換
                            string requestDiv = row.Field<string>("request_div").ToString();
                            if ("1".Equals(requestDiv))
                            {
                                row.SetField<string>("request_div", "選任");

                            }
                            else if ("2".Equals(requestDiv))
                            {
                                row.SetField<string>("request_div", "解任");
                            }

                            //申請ステータス（コード）を名称に変換
                            string applovalFlag = lstApprovalFlag[row.Field<int>("apploval_flag").ToString()];

                            if (!string.IsNullOrEmpty(applovalFlag))
                            {
                                row.SetField<string>("ステータス", applovalFlag);
                            }
                        }

                        //検索結果をセッションに設定
                        Session[SESSION_SELECTION_CONDITION_SEARCH_RESULT] = resultTable;

                        gvBumonSelectionList.DataSource = resultTable;
                        gvBumonSelectionList.DataBind();

                        if (resultTable.Rows.Count == 0)
                        {
                            lblErrMsg.Text = "選択された従事者の選任解任情報が存在しません。";
                        }
                    }
                }
                else //チェックされている行が存在しない場合
                {
                    lblErrMsg.Text = "従事者情報が選択されていません。";
                }
            }
            catch (Exception ex)
            {
                lblErrMsg.Text = ex.Message;
                throw ex;
            }
        }
        #endregion 選任解任状況検索・表示処理

        //従事者情報一覧ページ変更イベント
        protected void gvWorker_PageChanging(object sender, GridViewPageEventArgs e)
        {
            gvWorkerSearchResult.PageIndex = e.NewPageIndex;
            gvWorkerSearchResult.PageSize = gvPagesize;
            gvWorkerSearchResult.DataSource = Session[SESSION_WORKER_SEARCH_RESULT] as DataTable;
            gvWorkerSearchResult.DataBind();
        }

        protected void gvBumonSelectionList_PageChanging(object sender, GridViewPageEventArgs e)
        {
            gvBumonSelectionList.PageIndex = e.NewPageIndex;
            gvBumonSelectionList.DataSource = Session[SESSION_SELECTION_CONDITION_SEARCH_RESULT] as DataTable;
            gvBumonSelectionList.DataBind();
        }

        protected void gvDetailHistory_PageChanging(object sender, GridViewPageEventArgs e)
        {
            gvDetailHistory.PageIndex = e.NewPageIndex;

            gvDetailHistory.DataSource = Session[SESSION_DETAIL_HISTORY_SEARCH_RESULT] as DataTable;
            gvDetailHistory.DataBind();
        }

        /// <summary>
        /// 文字列セルを生成する共通メソッド
        /// </summary>
        private TableCell createTextCell(string text)
        {
            TableCell cell = new TableCell();
            cell.Text = HttpUtility.HtmlEncode(text);
            return cell;
        }
        /// <summary>
        /// 「選任/解任詳細」ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void BtnSelectionDetail_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + INFO_LOG_SELECTIONDETAILBUTTON_PUSH);

            Session[SESSION_DETAIL_HISTORY_SEARCH_RESULT] = null;
            Session[SESSION_SELECTED_SELECTION_CONDITION_ROW] = null;
            lblErrMsg.Text = string.Empty;

            DataTable dtSelectionCondition= Session[SESSION_SELECTION_CONDITION_SEARCH_RESULT] as DataTable;

           //チェック状態を取得
            foreach (GridViewRow gvRow in gvBumonSelectionList.Rows)
            {
                if ((gvRow.Cells[0].FindControl("radioSelectCondition") as RadioButton).Checked)
                {
                    //チェックされた行をセッションに設定
                    Session[SESSION_SELECTED_SELECTION_CONDITION_ROW] = dtSelectionCondition.Rows[gvRow.DataItemIndex] as DataRow;
                }
            }

            try
            {
                // 選任解任詳細情報検索・表示処理
                SearchSelectionDetail();

                // 従事者情報一覧を表示
                PnlWorkerSearchResult.Visible = true;
                //部門毎選任解任状況一覧を表示
                PnlBumonSelectionCondition.Visible = true;
                // 選任解任詳細情報を表示
                //PnlSelectionDetail.Visible = true;
            }
            catch (Exception ex)
            {
                lblErrMsg.Text = ex.Message;
                logger.Error("User:" + userRole.UserId + ex.Message, ex);
            }
            finally
            {
                logger.Info(ComConst.INFO_LOG_END + userRole.UserId + INFO_LOG_SELECTIONDETAILBUTTON_PUSH);
            }
        }

        #region SearchSelectionDetail 選任解任詳細情報検索・表示
        private void SearchSelectionDetail()
        {
            try
            {
                //選択された従事者情報を取得
                DataRow checkedPerson = Session[SESSION_SELECTED_WORKER_ROW] as DataRow;
                //選択された選任解任状況を取得
                DataRow checkedSelectionCondition = Session[SESSION_SELECTED_SELECTION_CONDITION_ROW] as DataRow;

                if(checkedPerson!=null && checkedSelectionCondition != null)
                {
                    string engineerId = checkedPerson.Field<string>("enginnerId");
                    string selectionId = checkedSelectionCondition.Field<string>("selectionId");

                    //選任解任状況を取得
                    getSelectionDetailInfo(engineerId, selectionId);
                    //選任解任履歴を取得
                    getSelectionHistory(engineerId, selectionId);
                    //画面表示処理
                    displaySelectionDetail();
                }
                else
                {
                    //ラジオボタンにチェックが入っていない場合
                    throw new Exception("部門が選択されていません。");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 詳細情報表示処理
        /// </summary>
        /// <returns></returns>
        private void displaySelectionDetail()
        {
            bool displayFlag = true;

            //セッションから情報を取得し、画面表示する。
            DataRow drWorker = Session[SESSION_SELECTED_WORKER_ROW] as DataRow;
            DataRow drSelection = Session[SESSION_SELECTED_SELECTION_CONDITION_ROW] as DataRow;
            var bumonDict = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string, string>;

            // 従事者情報表示
            lblDetailBumon.Text = HttpUtility.HtmlEncode(m_ComFunc.GetSelectionSectionName(drSelection.Field<string>("selectionID"), bumonDict));
            lblDetailLastNameKana.Text = HttpUtility.HtmlEncode(drWorker.Field<string>("LastName_Kana"));
            lblDetailFirstNameKana.Text = HttpUtility.HtmlEncode(drWorker.Field<string>("FirstName_Kana"));
            //lblDetailBirthDay.Text = HttpUtility.HtmlEncode(drWorker.Field<DateTime>("birthday").ToString(DATE_FORMAT));
            lblDetailLicenseNumber.Text = HttpUtility.HtmlEncode(drWorker.Field<string>("license_number"));
            lblDetailLastName.Text = HttpUtility.HtmlEncode(drWorker.Field<string>("LastName"));
            lblDetailFirstName.Text = HttpUtility.HtmlEncode(drWorker.Field<string>("FirstName"));
            //lblDetailGetLicenseDate.Text = HttpUtility.HtmlEncode(drWorker.Field<DateTime>("getlicense_date").ToString(DATE_FORMAT));

            //選任解任詳細表示
            DataTable dt = Session[SESSION_DETAIL_SELECTION_INFO] as DataTable;
            if (dt.Rows.Count > 0)
            {
                //選任状況
                var selectionDateInfo = dt.AsEnumerable().Where(row => "1".Equals(row.Field<string>("request_div"))).OrderByDescending(row => row.Field<int>("area_code"));


                DataTable dtSelectionDate = dt.Clone();
                for (int i = 11; i >= 1; i--)
                {
                    if (selectionDateInfo.AsEnumerable().Any(row => i.Equals(row.Field<int>("area_code"))))
                    {
                        //該当地域コードのレコードが存在する場合
                        DataRow newrow = dtSelectionDate.NewRow();
                        newrow.ItemArray = selectionDateInfo.AsEnumerable().Where(row => i.Equals(row.Field<int>("area_code"))).Single().ItemArray;
                        dtSelectionDate.Rows.Add(newrow);
                    }
                    else
                    {
                        //該当地域コードのレコードが存在しない場合
                        DataRow newrow = dtSelectionDate.NewRow();
                        newrow.ItemArray = dt.Rows[0].ItemArray;
                        newrow.SetField<int>("area_code", i);
                        newrow.SetField<DateTime?>("sr_date", null);
                        dtSelectionDate.Rows.Add(newrow);
                    }
                }

                TableRow selectedRow = new TableRow();
                //空セルを追加
                TableCell selectDateCell = new TableCell();
                selectDateCell.Text = "選任日";
                selectDateCell.CssClass = "RowHeaderCell";
                selectedRow.Controls.Add(selectDateCell);

                foreach (DataRow row in dtSelectionDate.Rows)
                {
                    if (!row.IsNull("sr_date"))
                    {
                        selectedRow.Controls.Add(createTextCell(row.Field<DateTime>("sr_date").ToString(DATE_FORMAT)));
                    }
                    else
                    {
                        selectedRow.Controls.Add(createTextCell(string.Empty));
                    }
                }

                tblSelectionDetailInfo.Controls.Add(selectedRow);


                //解任状況
                var dismissDateInfo = dt.AsEnumerable().Where(row => "2".Equals(row.Field<string>("request_div"))).OrderByDescending(row => row.Field<int>("area_code"));

                DataTable dtDismissDate = dt.Clone();
                for (int i = 11; i >= 1; i--)
                {
                    if (dismissDateInfo.AsEnumerable().Any(row => i.Equals(row.Field<int>("area_code"))))
                    {
                        //該当地域コードのレコードが存在する場合
                        DataRow newrow = dtDismissDate.NewRow();
                        newrow.ItemArray = dismissDateInfo.AsEnumerable().Where(row => i.Equals(row.Field<int>("area_code"))).Single().ItemArray;
                        dtDismissDate.Rows.Add(newrow);
                    }
                    else
                    {
                        //該当地域コードのレコードが存在しない場合
                        DataRow newrow = dtDismissDate.NewRow();
                        newrow.ItemArray = dt.Rows[0].ItemArray;
                        newrow.SetField<int>("area_code", i);
                        newrow.SetField<DateTime?>("sr_date", null);
                        dtDismissDate.Rows.Add(newrow);
                    }
                }


                TableRow dismissRow = new TableRow();
                //空セルを追加
                TableCell dismissDateCell = new TableCell();
                dismissDateCell.Text = "解任日";
                dismissDateCell.CssClass = "RowHeaderCell";
                dismissRow.Controls.Add(dismissDateCell);

                foreach (DataRow row in dtDismissDate.Rows)
                {
                    //dismissRow.Controls.Add(createTextCell(row.Field<DateTime>("sr_date").ToString(DATE_FORMAT)));
                    if (!row.IsNull("sr_date"))
                    {
                        dismissRow.Controls.Add(createTextCell(row.Field<DateTime>("sr_date").ToString(DATE_FORMAT)));
                    }
                    else
                    {
                        dismissRow.Controls.Add(createTextCell(string.Empty));
                    }
                }

                tblSelectionDetailInfo.Controls.Add(dismissRow);
            }
            else
            {
                lblErrMsg.Text = "選任解任情報が存在しません。";
                displayFlag = false;
            }

            ////選任解任履歴表示
            gvDetailHistory.DataSource = Session[SESSION_DETAIL_HISTORY_SEARCH_RESULT] as DataTable;
            gvDetailHistory.DataBind();

            PnlSelectionDetail.Visible = true;
            pnlDetailChild.Visible = displayFlag;
        }

        /// <summary>
        /// 選任解任詳細情報を取得する。
        /// </summary>
        /// <param name="engineerId"></param>
        /// <param name="selectionId"></param>
        private void getSelectionDetailInfo(string engineerId,string selectionId)
        {
            //string strSql = string.Empty;
            DataTable resultTable = new DataTable();

            try
            {
                string strSql = string.Empty;
                strSql += "select";
                strSql += " psn.enginnerID,";
                strSql += " psn.selectionID,";
                strSql += " psn.belong_company,";
                strSql += " psn.request_date,";
                strSql += " psn.request_div,";
                strSql += " dt.area_code,";
                strSql += " dt.sr_date";
                strSql += " from selection_personal_tbl psn";
                strSql += " inner join selection_date_tbl dt";
                strSql += " on psn.enginnerID = dt.enginnerID";
                strSql += " and psn.selectionID = dt.selectionID";
                strSql += " and psn.request_date = dt.request_date";
                //strSql += " and psn.request_div = dt.sr_flag";
                strSql += " where ";
                strSql += " psn.enginnerID = @enginnerID";
                strSql += " and psn.selectionID = @selectionID";

                List<SqlParameter> lstParam = new List<SqlParameter>();
                lstParam.Add(new SqlParameter("@enginnerID", engineerId));
                lstParam.Add(new SqlParameter("@selectionID", selectionId));

                if (m_ComFunc.DB_Select(strSql, lstParam, ref resultTable))
                {
                    //SQL正常実行時
                    Session[SESSION_DETAIL_SELECTION_INFO] = resultTable;
                }
                else
                {
                    //SQL異常終了時
                    throw new Exception("検索できませんでした。");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        //選任解任履歴を取得する。
        private void getSelectionHistory(string engineerId,string selectionId)
        {
            string strSQL = string.Empty;
            DataTable resultTable = new DataTable();
            try
            {
                string strSelect = string.Empty;
                strSelect = "select hokkaidou,touhoku,hokuriku,shinetsu,kantou,tokai,kinki,shikoku,chugoku,kyuushu,okinawa,request_date,belong_company";
                strSelect += " from selection_personal_tbl ";

                string strWhere = string.Empty;

                strWhere = " where enginnerID= @enginnerID and selectionID = @selectionID and delete_flag =0 ";
                string strOrder = " order by request_date desc";
                strSQL = strSelect + strWhere + strOrder;

                List<SqlParameter> lstParam = new List<SqlParameter>();
                lstParam.Add(new SqlParameter("@enginnerID",engineerId));
                lstParam.Add(new SqlParameter("@selectionID", selectionId));

                //SQL実行
                if (m_ComFunc.DB_Select(strSQL, lstParam, ref resultTable))
                {
                    //SQL正常実行完了時

                    resultTable.Columns.Add("num");
                    resultTable.Columns.Add("disp_request_date",typeof(string));
                    int i=1;
                    foreach (DataRow row in resultTable.Rows)
                    {
                        //連番列を追加
                        row.SetField<string>("num", i++.ToString());
                        //申請日を入れなおし
                        row.SetField<string>("disp_request_date", row.Field<DateTime>("request_date").ToString("yyyy/MM/dd"));
                    }

                    //検索結果をセッションに設定
                    Session[SESSION_DETAIL_HISTORY_SEARCH_RESULT] = resultTable;
                }
                else
                {
                    //SQL異常終了時
                    throw new Exception("選任解任履歴が取得できませんでした。");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }


        #endregion SearchSelectionDetail 選任解任詳細情報検索・表示

        //選任解任検索編集画面に遷移する
        protected void EditSelection_click(object sender, EventArgs e)
        {
            Session[ComConst.SELECTION_ID] = (Session[SESSION_SELECTED_SELECTION_CONDITION_ROW] as DataRow).Field<string>("selectionID");
            Session[ComConst.LICENSE_NUM] = (Session[SESSION_SELECTED_WORKER_ROW] as DataRow).Field<string>("license_number");
            Session[ComConst.LASTNAME] = (Session[SESSION_SELECTED_WORKER_ROW] as DataRow).Field<string>("LastName");
            Session[ComConst.FIRSTNAME] = (Session[SESSION_SELECTED_WORKER_ROW] as DataRow).Field<string>("FirstName");
            Session[ComConst.LASTNAME_KANA] = (Session[SESSION_SELECTED_WORKER_ROW] as DataRow).Field<string>("LastName_Kana");
            Session[ComConst.FIRSTNAME_KANA] = (Session[SESSION_SELECTED_WORKER_ROW] as DataRow).Field<string>("FirstName_Kana");

            //遷移前に不要なセッションをクリアする。
            Session[SESSION_DETAIL_HISTORY_SEARCH_RESULT] = null;
            Session[SESSION_DETAIL_SELECTION_INFO] = null;
            Session[SESSION_WORKER_SEARCH_RESULT] = null;
            Session[SESSION_SELECTED_WORKER_ROW] = null;

            Response.Redirect("EmployeeSearch.aspx", false);
        }

        protected void ClearBtn_Click(object sender, EventArgs e)
        {
            //入力内容をクリアする
            licenseno.Text = string.Empty;
            lastname.Text = string.Empty;
            firstname.Text = string.Empty;
            furi_familyname.Text = string.Empty;
            furi_firstname.Text = string.Empty;

            ddlBelongCompany.SelectedIndex = 0;
            ddlBelongCompany_Changed(sender, e);
            
            PnlBumonSelectionCondition.Visible = false;
            PnlSelectionDetail.Visible = false;
            PnlWorkerSearchResult.Visible = false;
        }

        protected void modoru_Click(object sender, EventArgs e)
        {
            Response.Redirect("Mainmenu.aspx", false);
        }
    }
}