﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using EmpCRM.common;
using EmpCRM.Dto;
using System.Data;
using System.Data.SqlClient;


namespace EmpCRM
{
    public partial class DepartmentChange : System.Web.UI.Page
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        //共通クラス
        private ComFunc m_ComFunc = new ComFunc();

        //GridViewページサイズ
        private static int gvPagesize = int.Parse(System.Configuration.ConfigurationManager.AppSettings[ComConst.CONFIG_GRIDVIEW_PAGESIZE]);

        #region イベント定義

        protected void Page_Load(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //ログイン状態チェック
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                lblErrMessage.Text = string.Empty;

                //開始ログ出力
                logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "Page_Load");
                if (!IsPostBack)
                {
                    //初期表示処理
                    try
                    {
                        //現在設定されている部門一覧を取得しDDLに設定
                        var lstCurrentSections = m_ComFunc.getCurrentSelectionSectionList();
                        foreach (string section in lstCurrentSections)
                        {
                            ddlCurrentBumon.Items.Add(section);
                        }
                        ddlCurrentBumon_Changed(ddlCurrentBumon, null);
                    }
                    catch (Exception ex)
                    {
                        logger.Error("User:" + userRole.UserId + ex.Message, ex);
                        lblErrMessage.Text += " 初期表示処理不正終了 ";
                    }
                }
                else
                {
                    //Postback時
                }
                //終了ログ出力
                logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "Page_Load");
            }
        }

        /// <summary>
        /// 部門変更実行ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnChangeDepartment_Click(object sender, EventArgs e)
        {
            UserRole user = Session[ComConst.SESSION_USER_ROLE] as UserRole;
            logger.Info(ComConst.INFO_LOG_START + user.UserId + "部門変更実行ボタン押下処理");

            lblErrMessage.Visible = false;
            pnlErrorData.Visible = false;
            //画面入力値を取得
            string currentBumon = ddlCurrentBumon.SelectedValue;
            string changeBumon = ccDepartmentSelector.GetSelectedValue();

            if (string.IsNullOrEmpty(changeBumon))
            {
                lblErrMessage.Text = "変更後の部門が選択されていません。";
                lblErrMessage.Visible = true;
                return;
            }

            try
            {
                //更新処理実行
                changeDepartmentExecute(currentBumon, changeBumon);

                lblErrMessage.Text = "部門の変更が完了しました。";
                lblErrMessage.Visible = true;
            }
            catch (Exception ex)
            {
                lblErrMessage.Text = "部門変更処理異常終了。下記データの変更に失敗しました。";
                lblErrMessage.Visible = true;

                //エラーデータ表示
                displayErrorData(currentBumon);

                logger.Error("User:" + user.UserId + ex.Message, ex);
            }

            logger.Info(ComConst.INFO_LOG_END + user.UserId + "部門変更実行ボタン押下処理");
        }


        protected void ddlCurrentBumon_Changed(object sender, EventArgs e)
        {
            string selectedSection = (sender as DropDownList).SelectedValue;
            Dictionary<string, string> dictBumon = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string, string>;
            if (dictBumon != null && dictBumon.Count > 0 && dictBumon.ContainsKey(selectedSection))
            {
                lblCurrentBumonName.Text = dictBumon[selectedSection];
            }
            else
            {
                lblCurrentBumonName.Text = "指定された部門名は現在取得できません。";
            }
        }

        protected void btnClear_Click(object sender, EventArgs e)
        {

            this.initializeControls();
        }

        protected void modoru_Click(object sender, EventArgs e)
        {
            Response.Redirect("Mainmenu.aspx", false);
        }
        #endregion イベント定義

        #region PrivateMethod
        private void changeDepartmentExecute(string currentBumon,string changeBumon)
        {
            string updatePersonalTableSql = string.Empty;
            string updateSectionDateTableSql = string.Empty;

            UserRole user = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //selection_personal_tbl 更新用SQL作成
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            sb.AppendLine(" update selection_personal_tbl ");
            sb.AppendLine(" set");
            sb.AppendLine("   selectionID = @newSelectionID");
            sb.AppendLine("   , update_date = SYSDATETIME()");
            sb.AppendLine("   , update_name = @update_name ");
            sb.AppendLine(" where");
            sb.AppendLine("   selectionID = @oldSelectionID");
            updatePersonalTableSql = sb.ToString();

            sb.Clear();
            //selection_date_tbl 更新用SQL作成
            sb.AppendLine(" update selection_date_tbl ");
            sb.AppendLine(" set");
            sb.AppendLine("   selectionID = @date_newSelectionID ");
            sb.AppendLine(" where");
            sb.AppendLine("   selectionID = @date_oldSelectionID");
            updateSectionDateTableSql = sb.ToString();

            //SqlParameter生成
            List<SqlParameter> lstUpdatePersonalParams = new List<SqlParameter>();
            List<SqlParameter> lstUpdateDateParams = new List<SqlParameter>();

            lstUpdatePersonalParams.Add(new SqlParameter("@oldSelectionID",currentBumon));
            lstUpdatePersonalParams.Add(new SqlParameter("@newSelectionID",changeBumon));
            lstUpdatePersonalParams.Add(new SqlParameter("@update_name", user.UserId));
            lstUpdateDateParams.Add(new SqlParameter("@date_oldSelectionID", currentBumon));
            lstUpdateDateParams.Add(new SqlParameter("@date_newSelectionID", changeBumon));

            try
            {
                //トランザクション開始
                m_ComFunc.DB_BeginTranzaction();

                m_ComFunc.DB_Execute(updatePersonalTableSql, lstUpdatePersonalParams);
                m_ComFunc.DB_Execute(updateSectionDateTableSql, lstUpdateDateParams);

                //コミット
                m_ComFunc.DB_Commit();
            }
            catch (Exception ex)
            {
                //ロールバック
                m_ComFunc.DB_Rollback();
                throw ex;
            }
        }

        private void displayErrorData(string selectionId)
        {
            string strSql = "";
            strSql += " select";
            strSql += " ROW_NUMBER() OVER(order by psn.enginnerID , psn.selectionID) 'No', ";
            strSql += "   psn.enginnerID";
            strSql += "   , psn.selectionID";
            strSql += "   , psn.request_date";
            strSql += "   , psn.belong_company";
            strSql += "   , psn.belong_department";
            strSql += "   , psn.jmcia_flag";
            strSql += "   , psn.sb_wcp";
            strSql += "   , eng.license_number";
            strSql += "   , eng.LastName";
            strSql += "   , eng.FirstName";
            strSql += "   , eng.LastName_Kana";
            strSql += "   , eng.FirstName_Kana ";
            strSql += " from";
            strSql += "   selection_personal_tbl psn ";
            strSql += "   inner join radio_enginner_info_tbl eng ";
            strSql += "     on eng.enginnerID = psn.enginnerID ";
            strSql += " where";
            strSql += "   selectionID = @selectionID";

            List<SqlParameter> lstSqlparam = new List<SqlParameter>();
            lstSqlparam.Add(new SqlParameter("@selectionID", selectionId));

            DataTable dt = new DataTable();

            try
            {
                m_ComFunc.DB_Select(strSql, lstSqlparam, ref dt);
            }
            catch (Exception ex)
            {
                //ロールバック
                throw ex;
            }

            dt.Columns.Add("sectionName", typeof(string));
            dt.Columns.Add("申請日", typeof(string));
            string sectionName = new RoleProvider().GetSectionName(selectionId);
            foreach (DataRow row in dt.Rows)
            {
                row.SetField<string>("sectionName", sectionName);
                row.SetField<string>("申請日", row.Field<DateTime>("request_date").ToString("yyyy/MM/dd"));
            }

            Session["ERROR_RESULT"] = dt;
            gvErrorData.DataSource = dt;
            gvErrorData.DataBind();

            pnlErrorData.Visible = true;
        }

        protected void gvPageChanging(object sender, GridViewPageEventArgs e)
        {
            gvErrorData.PageIndex = e.NewPageIndex;
            gvErrorData.PageSize = gvPagesize;
            gvErrorData.DataSource = Session["ERROR_RESULT"] as DataTable;
            gvErrorData.DataBind();
        }

        private void initializeControls()
        {
            Session["ERROR_RESULT"] = null;
            ddlCurrentBumon.Items.Clear();
            //現在設定されている部門一覧を取得しDDLに設定
            var lstCurrentSections = m_ComFunc.getCurrentSelectionSectionList();
            foreach (string section in lstCurrentSections)
            {
                ddlCurrentBumon.Items.Add(section);
            }
            ddlCurrentBumon_Changed(ddlCurrentBumon, null);

            ccDepartmentSelector.InitializeControls();
            lblErrMessage.Text = string.Empty;

            pnlErrorData.Visible = false;
        }
        #endregion PrivateMethod
    }
}