﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Data.SqlClient;
using EmpCRM.common;
using EmpCRM.Dto;

namespace EmpCRM
{
    public partial class EmployeeSelectedDateAdd : System.Web.UI.Page
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private ComFunc m_ComFunc = new ComFunc();
        private Dictionary<int, string> dicArea = new Dictionary<int, string>();
        
        private string strSelectionDate = string.Empty;
        private string strDismissDate = string.Empty;
        private string strArea = string.Empty;
        private string strSbwcp = string.Empty;

        #region イベント定義
        protected void Page_Load(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //ログイン判定
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                if (!IsPostBack)
                {
                    //画面入力項目を初期化
                    start_date.Text = strSelectionDate;
                    end_date.Text = strDismissDate;
                    ddlArea.SelectedIndex = 0;
                    ddlSbwcp.SelectedIndex = 0;

                    //本部の場合、全地域を表示する。
                    if (userRole.Areas.Contains("HQ"))
                    {
                        dicArea = createDicArea();
                    }
                    else
                    {
                        //本部以外の場合、ロールに設定された地域のみを取得する。
                        foreach (string area in userRole.Areas)
                        {
                            //地域リストに追加済みで無ければ追加する。
                            if (!dicArea.ContainsKey(m_ComFunc.getAreaCode(area)))
                            {
                                dicArea.Add(m_ComFunc.getAreaCode(area), m_ComFunc.GetAreaName(area));
                            }
                        }
                    }

                    //ddlArea.Items.Add(new ListItem(string.Empty, string.Empty));
                    foreach (var item in dicArea.OrderBy(m => m.Key))
                    {
                        ListItem dt = new ListItem(item.Value, item.Key.ToString());
                        ddlArea.Items.Add(dt);
                    }

                    int i = 0;
                    ddlSbwcp.Items.Add(new ListItem(string.Empty, string.Empty));
                    var lstSbwcp = m_ComFunc.GetSbWcpList();
                    foreach (KeyValuePair<string,string> pair in lstSbwcp)
                    {
                        ListItem dt = new ListItem(pair.Value, pair.Key);
                        ddlSbwcp.Items.Add(dt);
                    }
                }
            }
        }

        /// <summary>
        /// 登録ボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void BtnRegist_Click(object sender, EventArgs e)
        {
            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "登録ボタン押下時");

            //入力値取得
            strSelectionDate = start_date.Text;
            strDismissDate = end_date.Text;
            strArea = ddlArea.SelectedValue;
            strSbwcp = ddlSbwcp.SelectedValue;

            //エラーメッセージ初期化
            lblErrMsg.Text = string.Empty;

            //入力値チェック
            //選任日/解任日のチェック
            //日付が入力されていること。
            if (string.IsNullOrEmpty(strSelectionDate) && string.IsNullOrEmpty(strDismissDate))
            {
                lblErrMsg.Text = "選任日、解任日が入力されていません。";
                lblErrMsg.Visible = true;
                return;
            }
        
            //入力された選任日で対象レコードを更新
            if (!string.IsNullOrEmpty(strSelectionDate))
            {
                List<SqlParameter> lstParam = new List<SqlParameter>();
                lstParam.Add(new SqlParameter("@selectionDate", DateTime.ParseExact(strSelectionDate, "yyyy/MM/dd", null)));

                string strUpdate = string.Empty;
                strUpdate += "UPDATE date ";
                strUpdate += " SET date.sr_date = @selectionDate";
                strUpdate += " FROM selection_date_tbl date";
                strUpdate += " INNER JOIN selection_personal_tbl ps";
                strUpdate += " ON date.enginnerID = ps.enginnerID ";
                strUpdate += " AND date.selectionID = ps.selectionID";
                strUpdate += " AND date.request_date = ps.request_date";
                //strUpdate += " AND date.sr_flag = ps.request_div";
                strUpdate += " WHERE date.sr_flag = 1";
                strUpdate += " AND ps.apploval_flag = 2";//承認済みのデータを更新
                strUpdate += " AND date.sr_date IS NULL";
                //地域が選択されていた場合
                if (!string.IsNullOrEmpty(strArea))
                {
                    strUpdate += " AND date.area_code = @area";
                    lstParam.Add(new SqlParameter("@area", int.Parse(strArea)));
                }

                //SB/WCPが選択されていた場合
                if (!string.IsNullOrEmpty(strSbwcp))
                {
                    strUpdate += " AND ps.sb_wcp = @sbwcp";
                    lstParam.Add(new SqlParameter("@sbwcp", int.Parse(strSbwcp)));
                }

                try
                {
                    // トランザクション開始
                    m_ComFunc.DB_BeginTranzaction();

                    //UPDATE実行
                    if (m_ComFunc.DB_Execute(strUpdate, lstParam))
                    {
                        //コミット
                        m_ComFunc.DB_Commit();

                        //正常系
                        lblErrMsg.Text = "選任日の更新が完了しました。";
                    }
                    else
                    {
                        //対象データなしの場合
                        //ロールバック
                        m_ComFunc.DB_Rollback();
                        lblErrMsg.Text += "更新対象の選任データが存在しません。";
                    }
                }
                catch (Exception ex) //例外発生時
                {
                    //ロールバック
                    m_ComFunc.DB_Rollback();
                    lblErrMsg.Text = ex.Message;
                    logger.Error("User:" + userRole.UserId + ex.Message, ex);
                }
            }

            //入力された解任日で対象レコードを更新
            if (!string.IsNullOrEmpty(strDismissDate))
            {
                List<SqlParameter> lstParam = new List<SqlParameter>();
                lstParam.Add(new SqlParameter("@dismissDate", DateTime.ParseExact(strDismissDate, "yyyy/MM/dd", null)));

                string strUpdate = string.Empty;
                strUpdate += "UPDATE date ";
                strUpdate += " SET date.sr_date = @dismissDate";
                strUpdate += " FROM selection_date_tbl date";
                strUpdate += " INNER JOIN selection_personal_tbl ps";
                strUpdate += " ON date.enginnerID = ps.enginnerID ";
                strUpdate += " AND date.selectionID = ps.selectionID";
                strUpdate += " AND date.request_date = ps.request_date";
                //strUpdate += " AND date.sr_flag = ps.request_div";
                strUpdate += " WHERE date.sr_flag = 2";
                strUpdate += " AND ps.apploval_flag = 2"; //承認済みデータを更新する
                strUpdate += " AND date.sr_date IS NULL";

                //地域が選択されていた場合
                if (!string.IsNullOrEmpty(strArea))
                {
                    strUpdate += " AND date.area_code = @area";
                    lstParam.Add(new SqlParameter("@area", int.Parse(strArea)));
                }

                //SB/WCPが選択されていた場合
                if (!string.IsNullOrEmpty(strSbwcp))
                {
                    strUpdate += " AND ps.sb_wcp = @sbwcp";
                    lstParam.Add(new SqlParameter("@sbwcp", int.Parse(strSbwcp)));
                }

                try
                {
                    // トランザクション開始
                    m_ComFunc.DB_BeginTranzaction();

                    //UPDATE実行
                    if (m_ComFunc.DB_Execute(strUpdate, lstParam))
                    {
                        //コミット
                        m_ComFunc.DB_Commit();

                        //正常系
                        lblErrMsg.Text += "解任日の更新が完了しました。";
                    }
                    else
                    {
                        //対象データ無しの場合
                        //ロールバック
                        m_ComFunc.DB_Rollback();
                        lblErrMsg.Text += "更新対象の解任データが存在しません。";
                    }
                }
                catch (Exception ex)
                {
                    //ロールバック
                    m_ComFunc.DB_Rollback();
                    lblErrMsg.Text = ex.Message;
                    logger.Error("User:" + userRole.UserId + ex.Message, ex);
                }
            }

            //エラーメッセージの表示判定
            if (!string.IsNullOrEmpty(lblErrMsg.Text))
            {
                lblErrMsg.Visible = true;
            }
            else
            {
                lblErrMsg.Visible = false;
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "登録ボタン押下時");
        }

        /// <summary>
        /// メニュー画面に戻る
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void modoru_Click(object sender, EventArgs e)
        {
            Response.Redirect("Mainmenu.aspx", false);
        }
        #endregion イベント定義


        private Dictionary<int, string> createDicArea()
        {
            Dictionary<int,string> retDict = new Dictionary<int,string>();
            string[] array = { "北海道", "東北", "北陸", "信越", "関東", "東海", "近畿", "四国", "中国", "九州", "沖縄" };
            int i = 1;
            foreach (string str in array)
            {
                retDict.Add(i++, str);
            }
            return retDict;
        }

    }
}