﻿//ActiveDirectory環境以外ではDEMOを定義してビルド
#define DEMO
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.IO;
using System.Configuration;
using System.Web.SessionState;
using EmpCRM.common;
using EmpCRM.Dto;
using System.Reflection;
using log4net;

namespace EmpCRM
{
    public partial class Login : System.Web.UI.Page
    {
        //ロガー生成
        private static ILog logger = LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        //共通関数クラス
        private ComFunc m_ComFunc = null;

        /// <summary>
        /// 初期化処理
        /// </summary>
        /// <param name="e"></param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            //セッション情報をクリア
            Session.Clear();
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            //初期画面の場合、
            if (!(this.IsPostBack))
            {
                txtID.Text = string.Empty;
                txtPass.Text = string.Empty;
            }
        }

        /// <summary>
        /// ログインボタン押下時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnLogin_Click(object sender, EventArgs e)
        {
            //初期化
            lblError.Text = string.Empty;

            string sUserID = string.Empty;
            string sPassword = string.Empty;

            if ((txtID.Text == string.Empty) || (txtPass.Text == string.Empty))
            {
                this.lblError.Text = "IDとパスワードを入力してください。";
                return;
            }
            sUserID = txtID.Text.Trim();
            sPassword = txtPass.Text.Trim();

            logger.Info(ComConst.INFO_LOG_START + sUserID + "ログインボタン押下処理");

#if DEMO
                //デモ時はユーザ情報を固定で設定する。
                UserRole userRole = new UserRole();
                List<string> lstAreas = new List<string>();
                List<string> lstRoles = new List<string>();
//                userRole.UserId = "test";
                
                //変更しました。
                userRole.UserId = sUserID;

                if ("system".Equals(sUserID))
                {
                    userRole.UserName = "システム部門ユーザ";
                    lstAreas.Add("HQ");
                    lstRoles.Add(ComConst.SYSTEM_OPERATE_USER);
                }
                else if ("request".Equals(sUserID))
                {
                    userRole.UserName = "申請ユーザ";
                    lstAreas.Add("A");
                    lstRoles.Add(ComConst.REQUEST_USER);
                }
                else if ("approval".Equals(sUserID))
                {
                    userRole.UserName = "承認ユーザ";
                    lstAreas.Add("B");
                    lstRoles.Add(ComConst.APPROVAL_USER);
                }
                userRole.Areas = lstAreas.ToArray();
                userRole.Roles = lstRoles.ToArray();

                userRole.DepartmentId = "1000";
                userRole.DepartmentName = "部門";

                userRole.LoginDateTime = DateTime.Now;
#else
            UserRole userRole = null;
            try
            {
                //ログイン処理実行
                var result = LoginUtil.ExecuteLogin(sUserID, sPassword);
                if (result != null)
                {
                    userRole = LoginUtil.GetUserRoleInfo(result,sUserID);
                }
                else
                {
                    lblError.Text = "ユーザ認証に失敗しました。";
                    return;
                }
            }
            catch (Exception ex)
            {
                logger.Error("User:" + sUserID + ex.Message, ex);
                lblError.Text = "ユーザ認証に失敗しました。";
                return;
            }
#endif
            try
            {
                if (userRole != null)
                {
                    //共通関数クラスのインスタンス生成
                    m_ComFunc = new ComFunc(sUserID);

                    try
                    {
                        //DBに接続
                        // Web.Configより接続情報を取得
                        var connString = ConfigurationManager.ConnectionStrings[ComConst.CONFIG_CONNECTION_STRING].ConnectionString;
                        if (!(m_ComFunc.DB_Connect(connString)))
                        {
                            lblError.Text = "DB接続に失敗しました。";
                            return;
                        }
                    }
                    catch (Exception ex)
                    {
                        logger.Error("User:" + sUserID + ex.Message, ex);
                        lblError.Text = "DB接続に失敗しました。";
                        return;
                    }
#if DEMO
                    //部門一覧をDBから取得
                    Dictionary<string, string> dictDepartment = new Dictionary<string, string>();
                    DataTable dtDepartment = new DataTable();
                    if (m_ComFunc.GetSelectionInfoMaster(ref dtDepartment))
                    {
                        //selectionID,selection_section 
                        foreach (DataRow row in dtDepartment.Rows)
                        {
                            dictDepartment.Add(row.Field<string>("selectionID"), row.Field<string>("selection_section"));
                        }
                    }
                    if (dictDepartment.Count == 0)
                    {
                        dictDepartment.Add(userRole.DepartmentId, userRole.DepartmentName);

                    }
#else
                    //部門一覧をActiveDirecotryから取得
                    var obj = new MembershipProvider();
                    var dictDepartment = obj.GetDepartmentList();
#endif

                    //セッション値を設定
                    Session[ComConst.SESSION_USER_ROLE] = userRole;
                    Session[ComConst.COM_FUNC] = m_ComFunc;
                    Session[ComConst.SESSION_BUMON_MASTER] = dictDepartment;

                    Response.Redirect("Mainmenu.aspx", false);
                }
                else
                {
                    lblError.Text = "ユーザ情報の取得に失敗しました。権限、部門が正しく設定されているか確認してください。";
                }
            }
            catch (Exception ex)
            {
                logger.Error("User:" + sUserID + ex.Message, ex);
                lblError.Text = "ログインに失敗しました。ログインしなおして下さい。";
            }
            finally
            {
                logger.Info(ComConst.INFO_LOG_END + sUserID + "ログインボタン押下処理");
            }
        }

    }
}