﻿#region " DBアクセス共通クラス"
//-----------------------------------------------------------------------
//【ファイル名】   DBアクセス共通クラス
//【機　能】      
//【備　考】  　　
//【作成者】      nakamura
//【作成日】      2017/1/28
//【修正履歴】    YYYY/MM/DD 修正内容　修正者
//-----------------------------------------------------------------------
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.SqlClient;
using System.Data;

namespace EmpCRM.common
{
    public class ComDB
    {
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region メンバー
        private SqlConnection m_Connection { get; set; }             //接続オブジェクト
        private SqlDataAdapter adpt = null;
        private SqlCommand m_Command = null;
        private SqlTransaction m_Transaction = null;              //OLEトランザクション

        #endregion

        #region コンストラクター
        public ComDB()
        {
            //
            // TODO: コンストラクター ロジックをここに追加します
            //
        }
        #endregion
        #region DBに接続する
        /// <summary>
        /// DBに接続する
        /// </summary>
        public void DB_Connect(string conStr)
        {
            try
            {
                m_Connection = new SqlConnection(conStr);

                //m_Connection.ConnectionString = connectString;
                m_Connection.Open();
            }
            catch (Exception ex)
            {
                m_Connection.Dispose();
                m_Connection.Close();

                throw ex;
            }
        }
        #endregion
        #region DBを切断する
        /// <summary>
        /// DBにs切断する。
        /// </summary>
        public void DB_Disconnect()
        {
            m_Connection.Close();
            m_Connection.Dispose();

        }
        #endregion
        #region Select文を発行する
        /// <summary>
        /// Select文を発行する
        /// </summary>
        public Boolean DB_Select(string psSql, ref SqlDataAdapter pSqlDataAdapter)
        {
            //SELECT文を発行

            //ログ出力
            logger.Info(LogUtil.GetSqlLogFormat(psSql));

            if (m_Command == null)
            {
                pSqlDataAdapter = new SqlDataAdapter(psSql, m_Connection);
            }
            else
            {
                m_Command.CommandText = psSql;
                pSqlDataAdapter = new SqlDataAdapter(m_Command);
            }
            return true;
        }
        #endregion

        #region SQLを実行する。
        /// <summary>
        /// 引数で渡されたSQLとSQLパラメータによりSelect文を実行する。
        /// </summary>
        /// <param name="strSql">プレースホルダを含むSelect文</param>
        /// <param name="sqlParams">SQLパラメータのリスト</param>
        /// <param name="resultTable">結果設定用DataTable</param>
        /// <returns>実行結果 True：正常終了、False：異常終了</returns>
        public bool DB_SelectByParams(string strSql, List<SqlParameter> sqlParams, ref DataTable resultTable)
        {
            SqlDataReader reader = null;

            try
            {
                // 実行用のSQLCommandを作成
                SqlCommand command = new SqlCommand(strSql, m_Connection);

                // SQLパラメータ設定
                foreach (var param in sqlParams)
                {
                    command.Parameters.Add(param);
                }

                try
                {
                    //ログ出力
                    logger.Info(LogUtil.GetSqlLogFormat(strSql, sqlParams));
                }
                catch(Exception ex)
                {
                    logger.Fatal(ex.Message, ex);
                }
                // Select実行
                reader = command.ExecuteReader();

                //実行結果が正常に取得できた場合
                if (reader != null && !reader.IsClosed)
                {
                    //DataTableに結果を設定
                    resultTable.Load(reader);
                }
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
                //return false;
            }
            finally
            {
                reader.Close();
            }
        }
        #endregion SQLを実行する。

        #region トランザクションを開始する
        /// <summary>
        /// トランザクションを開始する
        /// </summary>
        public void DB_BeginTranzaction()
        {
            m_Transaction = m_Connection.BeginTransaction();
            m_Command = m_Connection.CreateCommand();
            m_Command.Transaction = m_Transaction;
        }
        #endregion
        #region INSERT,DELETE,UPDATEを発行する
        /// <summary>
        /// INSERT,DELETE,UPDATEを発行する
        /// </summary>
        public void DB_Execute(string psSql)
        {
            //ログ出力
            logger.Info(LogUtil.GetSqlLogFormat(psSql));
            m_Command.CommandText = psSql;
            m_Command.ExecuteNonQuery();
        }
        #endregion

        public int DB_ExecuteByParams(string psSql, List<SqlParameter> sqlParams)
        {
            //ログ出力
            logger.Info(LogUtil.GetSqlLogFormat(psSql, sqlParams));

            m_Command.CommandText = psSql;
            foreach (SqlParameter param in sqlParams)
            {
                m_Command.Parameters.Add(param);
            }
            return m_Command.ExecuteNonQuery();
        }

        #region コミットする
        /// <summary>
        /// コミットする
        /// </summary>
        public void DB_Commit()
        {

            if (m_Transaction != null)
            {
                m_Transaction.Commit();
                m_Transaction.Dispose();
                m_Command = null;
                m_Transaction = null;
            }

        }
        #endregion
        #region ロールバックする
        /// <summary>
        /// ロールバックする
        /// </summary>
        public void DB_Rollback()
        {
            if (m_Transaction != null)
            {
                m_Transaction.Rollback();
                m_Transaction.Dispose();
                m_Command = null;
                m_Transaction = null;
            }
        }
        #endregion


    }
}