﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EmpCRM.Dto;
using System.Reflection;
using System.IO;
using log4net;
using System.DirectoryServices;

namespace EmpCRM.common
{
    public static class LoginUtil
    {
        private static readonly ILog logger = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        #region ログイン処理
        public static SearchResult ExecuteLogin(string userId, string password)
        {
            try
            {
                //パラメータで渡されたID/パスの組み合わせでLDAP認証を実行
                MembershipProvider provider = new MembershipProvider();
                
                SearchResult result = provider.ValidateUerAndGetUserInfo(userId, password);

                //if (provider.ValidateUser(userId, password))
                if(result!=null)
                {
                    //ログイン成功
                    System.Diagnostics.Debug.Print("ログイン成功");
                    //return true;
                    return result;
                }
                else
                {
                    //ログイン失敗
                    System.Diagnostics.Debug.Print("ユーザ認証失敗");
                    //return false;
                    return null;
                }
            }
            catch (Exception ex)
            {
                logger.Error(ex.Message, ex);
                throw ex;
            }
        }

        public static UserRole GetUserRoleInfo(SearchResult result,string userId)
        {
            MembershipProvider provider = new MembershipProvider();
            UserRole userRole = new UserRole();

            try
            {
                //部門コードを取得
                string departCode = provider.GetUserDepartment(result);
                userRole.DepartmentId = departCode;

                //部門コードから部門名を取得
                if (!string.IsNullOrEmpty(departCode))
                {
                    string departName = provider.GetDepartmentName(departCode);
                    userRole.DepartmentName = departName;
                }

                //ユーザIDを設定
                userRole.UserId = userId;

                //ユーザ名を取得
                userRole.UserName = provider.GetUserDisplayName(result);

                RoleProvider role = new RoleProvider();
                //ROLE情報を取得
                string[] roles = role.GetRolesForUser(result);
                userRole.Roles = roles;

                //地域情報を取得
                string[] areas = role.GetAreasForUser(result);
                userRole.Areas = areas.ToList().OrderBy(m => m).ToArray();

                //ログイン日時を設定
                userRole.LoginDateTime = DateTime.Now;

            }
            catch (Exception ex)
            {
                logger.Error(ex.Message, ex);
                throw ex;
            }

            //情報の取得に失敗したらログイン失敗とし、NULLを返す
            if (userRole.IsLogin())
            {
                return userRole;
            }
            else
            {
                logger.Info("ユーザー情報取得失敗：取得できていないユーザ情報があります。");
                return null;
            }
        }



        //ログイン判定
        public static bool IsLogin(UserRole role)
        {
            if (role != null && role.IsLogin())
            {
                //ログイン状態正常
                return true;
            }
            else
            {
                return false;
            }
        }

        #endregion ログイン処理


    }
}