﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.DirectoryServices;
using System.Reflection;
using log4net;

namespace EmpCRM.common
{
    public class RoleProvider : System.Web.Security.RoleProvider
    {
        //logger生成
        private static readonly ILog logger = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// パラメータで指定されたユーザの権限を取得する。
        /// </summary>
        /// <param name="username">ユーザID</param>
        /// <returns>権限</returns>
        public override string[] GetRolesForUser(string username)
        {
            logger.Info("START:" + username + ":ユーザ権限取得処理");

            string adminID = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_USER];
            string adminPW = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_PASSWORD];

            string domain = ConfigurationManager.AppSettings[ComConst.CONFIG_DOMAIN_NAME];
            try
            {
                using (DirectoryEntry entry = new DirectoryEntry())
                {
                    entry.Username = String.Format(@"{0}\{1}", domain, adminID);
                    entry.Password = adminPW;
                    try
                    {
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_PRIMARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }
                    catch (Exception ex)
                    {
                        logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                        logger.Error(adminID + ":プライマリサーバでの認証失敗", ex);

                        // サーバー1が駄目な場合、サーバー2に接続 
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_SECONDARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }

                    using (DirectorySearcher seacher = new DirectorySearcher(entry))
                    {
                        seacher.Filter = "(sAMAccountName=" + username + ")";
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_CN);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_MEMBEROF);

                        string props=string.Empty;
                        foreach (var item in seacher.PropertiesToLoad)
                        {
                            props += " "+item;
                        }
                        logger.Debug(string.Format("Filter:{0}, Properties:{1}", seacher.Filter, props.Trim()));
                        SearchResult result = seacher.FindOne();

                        if (result == null)
                        {
                            return new string[0];
                        }
                        else
                        {
                            List<string> directoryRoles = new List<string>();
                            foreach (string role in result.Properties[ComConst.ATTRIBUTE_MEMBEROF])
                            {
                                try
                                {
                                    logger.Debug(string.Format("User:{0}, ADからの取得情報 memberOf:{1}", username, role));
                                    if (role.Contains(ConfigurationManager.AppSettings[ComConst.CONFIG_APPLICATION_NAME]))
                                    {
                                        string[] arrayUnits = role.Split(',');
                                        string strOu = arrayUnits.First().Replace("CN=", string.Empty);
                                        if (!string.IsNullOrEmpty(strOu))
                                        {
                                            directoryRoles.Add(strOu);
                                            logger.Debug(string.Format("権限ロール：{0}を設定", strOu));
                                        }
                                    }
                                }
                                catch (Exception ex)
                                {
                                    throw ex;
                                }
                            }
                            return directoryRoles.ToArray();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                logger.Error(username + ":セカンダリサーバでの認証失敗", ex);
                throw ex;
            }
            finally
            {
                logger.Info("END  :" + username + ":ユーザ権限取得処理");
            }
        }

        public string[] GetRolesForUser(SearchResult result)
        {
            //logger.Info("START:" + username + ":ユーザ権限取得処理");
            logger.Info("START:ユーザ権限取得処理");

            try
            {
                List<string> directoryRoles = new List<string>();
                foreach (string role in result.Properties[ComConst.ATTRIBUTE_MEMBEROF])
                {
                    try
                    {
                        if (role.Contains(ConfigurationManager.AppSettings[ComConst.CONFIG_APPLICATION_NAME]))
                        {
                            string[] arrayUnits = role.Split(',');
                            string strOu = arrayUnits.First().Replace("CN=", string.Empty);
                            if (!string.IsNullOrEmpty(strOu))
                            {
                                switch (strOu)
                                {
                                    case ComConst.SYSTEM_OPERATE_USER:
                                    case ComConst.APPROVAL_USER:
                                    case ComConst.REQUEST_USER:
                                    directoryRoles.Add(strOu);
                                    logger.Debug(string.Format("権限ロール：{0}を設定", strOu));
                                    break;

                                    default:
                                        break;
                                }

                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }
                logger.Debug(string.Format("{0}件の権限ロールを設定", directoryRoles.Count.ToString()));
                return directoryRoles.ToArray();
            }
            catch (Exception ex)
            {
                logger.Debug(string.Format("権限ロール取得失敗"));
                logger.Error(ex.Message, ex);
                throw ex;
            }
            finally
            {
                //logger.Info("END  :" + username + ":ユーザ権限取得処理");
                logger.Info("END  :ユーザ権限取得処理");
            }
        }

        /// <summary>
        /// パラメータで指定されたユーザの地域ロールを取得する。
        /// </summary>
        /// <param name="result"></param>
        /// <returns>地域ロール</returns>
        public string[] GetAreasForUser(SearchResult result)
        {
            //logger.Info("START:" + username + ":ユーザ地域ロール取得処理");
            logger.Info("START:ユーザ地域ロール取得処理");

            try
            {
                List<string> directoryRoles = new List<string>();

                System.Text.RegularExpressions.Regex regex = new System.Text.RegularExpressions.Regex("^([ABCDEFGHIJO]|HQ)$");

                foreach (string role in result.Properties[ComConst.ATTRIBUTE_MEMBEROF])
                {
                    try
                    {
                        //logger.Debug(string.Format("User:{0}, ADからの取得情報 memberOf:{1}", username, role));
                        if (role.Contains(ConfigurationManager.AppSettings[ComConst.CONFIG_RO_AREA_NAME]))
                        {
                            string[] arrayUnits = role.Split(',');
                            string strOu = arrayUnits.First().Replace("CN=", string.Empty);
                            if (!string.IsNullOrEmpty(strOu))
                            {
                                string areaRole = ReplaceAreaCode(strOu.Replace(ComConst.RO_PREFIX, string.Empty));
                                logger.Debug(string.Format("地域ロール：{0}を取得", strOu));

                                //重複追加とならなければ追加
                                if (!directoryRoles.Contains(areaRole))
                                {
                                    //地域ロールに該当する場合は追加
                                    if (regex.IsMatch(areaRole))
                                    {
                                        directoryRoles.Add(areaRole);
                                        logger.Debug(string.Format("地域ロール：{0}を設定", areaRole));
                                    }
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }

                logger.Debug(string.Format("{0}件の地域ロールを設定", directoryRoles.Count.ToString()));

                return directoryRoles.ToArray();
            }
            catch (Exception ex)
            {
                //logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                //logger.Error(adminID + ":セカンダリサーバでの認証失敗", ex);
                logger.Debug(string.Format("地域ロール取得失敗"));
                logger.Error(ex.Message, ex);
                throw ex;
            }
            finally
            {
                //logger.Info("END  :" + username + ":ユーザ地域ロール取得処理");
                logger.Info("END  :ユーザ地域ロール取得処理");
            }
        }

        public string GetGroupName(string ouID)
        {
            //logger.Info("START:" + username + ":ユーザ権限取得処理");

            string adminID = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_USER];
            string adminPW = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_PASSWORD];

            string domain = ConfigurationManager.AppSettings[ComConst.CONFIG_DOMAIN_NAME];
            try
            {
                using (DirectoryEntry entry = new DirectoryEntry())
                {
                    entry.Username = String.Format(@"{0}\{1}", domain, adminID);
                    entry.Password = adminPW;
                    try
                    {
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_PRIMARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }
                    catch (Exception ex)
                    {
                        logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                        logger.Error(adminID + ":プライマリサーバでの認証失敗", ex);

                        // サーバー1が駄目な場合、サーバー2に接続 
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_SECONDARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }

                    using (DirectorySearcher seacher = new DirectorySearcher(entry))
                    {
                        seacher.Filter = "(&(objectClass=group)(name="+ouID+"))";
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_DISPLAYNAME);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_CN);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_MEMBEROF);

                        string props = string.Empty;
                        foreach (var item in seacher.PropertiesToLoad)
                        {
                            props += " " + item;
                        }
                        logger.Debug(string.Format("Filter:{0}, Properties:{1}", seacher.Filter, props.Trim()));
                        SearchResult result = seacher.FindOne();

                        if (result == null)
                        {
                            //取得失敗
                            return string.Empty;
                        }
                        else
                        {
                            foreach (string name in result.Properties[ComConst.ATTRIBUTE_DISPLAYNAME])
                            {
                                try
                                {
                                    //displayNameが設定されている場合
                                    if (!string.IsNullOrEmpty(name))
                                    {
                                        return name;
                                    }
                                }
                                catch (Exception ex)
                                {
                                    throw ex;
                                }
                            }

                            //displayNameが設定されていない場合はCNを返却
                            foreach (string name in result.Properties[ComConst.ATTRIBUTE_CN])
                            {
                                if (!string.IsNullOrEmpty(name))
                                {
                                    return name;
                                }
                            }

                            return string.Empty;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                //logger.Error(username + ":セカンダリサーバでの認証失敗", ex);
                throw ex;
            }
            finally
            {
                //logger.Info("END  :" + username + ":ユーザ権限取得処理");
            }
        }


        public string GetSectionName(string sectionId)
        {
            //logger.Info("START:" + username + ":ユーザ権限取得処理");

            string adminID = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_USER];
            string adminPW = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_PASSWORD];

            string domain = ConfigurationManager.AppSettings[ComConst.CONFIG_DOMAIN_NAME];
            try
            {
                using (DirectoryEntry entry = new DirectoryEntry())
                {
                    entry.Username = String.Format(@"{0}\{1}", domain, adminID);
                    entry.Password = adminPW;
                    try
                    {
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_PRIMARY_AUTH_SERVER] + ConfigurationManager.AppSettings[ComConst.CONFIG_SECTION_FILTER];
                        object adsiObj = entry.NativeObject;
                    }
                    catch (Exception ex)
                    {
                        logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                        logger.Error(adminID + ":プライマリサーバでの認証失敗", ex);

                        // サーバー1が駄目な場合、サーバー2に接続 
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_SECONDARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }

                    using (DirectorySearcher seacher = new DirectorySearcher(entry))
                    {
                        //seacher.Filter = "(&(objectClass=group)(name=" + ouID + "))";
                        seacher.Filter = "((name=" + sectionId + "))";
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_DISPLAYNAME);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_CN);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_MEMBEROF);

                        string props = string.Empty;
                        foreach (var item in seacher.PropertiesToLoad)
                        {
                            props += " " + item;
                        }
                        logger.Debug(string.Format("Filter:{0}, Properties:{1}", seacher.Filter, props.Trim()));
                        SearchResult result = seacher.FindOne();

                        if (result == null)
                        {
                            //取得失敗
                            return string.Empty;
                        }
                        else
                        {
                            foreach (string name in result.Properties[ComConst.ATTRIBUTE_DISPLAYNAME])
                            {
                                try
                                {
                                    //displayNameが設定されている場合
                                    if (!string.IsNullOrEmpty(name))
                                    {
                                        return name;
                                    }
                                }
                                catch (Exception ex)
                                {
                                    throw ex;
                                }
                            }

                            //displayNameが設定されていない場合はCNを返却
                            foreach (string name in result.Properties[ComConst.ATTRIBUTE_CN])
                            {
                                if (!string.IsNullOrEmpty(name))
                                {
                                    return name;
                                }
                            }

                            return string.Empty;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                //logger.Error(username + ":セカンダリサーバでの認証失敗", ex);
                throw ex;
            }
            finally
            {
                //logger.Info("END  :" + username + ":ユーザ権限取得処理");
            }
        }

        public Dictionary<string, string> SearchSections(string sectionName)
        {
            //logger.Info("START:" + username + ":ユーザ権限取得処理");

            string adminID = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_USER];
            string adminPW = ConfigurationManager.AppSettings[ComConst.CONFIG_ADMIN_PASSWORD];

            string domain = ConfigurationManager.AppSettings[ComConst.CONFIG_DOMAIN_NAME];

            Dictionary<string, string> departDict = new Dictionary<string, string>();

            try
            {
                using (DirectoryEntry entry = new DirectoryEntry())
                {
                    entry.Username = String.Format(@"{0}\{1}", domain, adminID);
                    entry.Password = adminPW;
                    try
                    {
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_PRIMARY_AUTH_SERVER] + ConfigurationManager.AppSettings[ComConst.CONFIG_SECTION_FILTER];
                        object adsiObj = entry.NativeObject;
                    }
                    catch (Exception ex)
                    {
                        logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                        logger.Error(adminID + ":プライマリサーバでの認証失敗", ex);

                        // サーバー1が駄目な場合、サーバー2に接続 
                        entry.Path = ConfigurationManager.AppSettings[ComConst.CONFIG_SECONDARY_AUTH_SERVER];
                        object adsiObj = entry.NativeObject;
                    }

                    using (DirectorySearcher seacher = new DirectorySearcher(entry))
                    {
                        //Web.Configの内容から検索対象の部署を絞り込む文字列を生成
                        string searchString = new common.ComFunc().GetFirstMatchSearchString();

                        seacher.Filter = "(&(displayName=*" + sectionName + "*)" + searchString + ")";
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_DISPLAYNAME);
                        seacher.PropertiesToLoad.Add(ComConst.ATTRIBUTE_OU);

                        string props = string.Empty;
                        foreach (var item in seacher.PropertiesToLoad)
                        {
                            props += " " + item;
                        }
                        logger.Debug(string.Format("Filter:{0}, Properties:{1}", seacher.Filter, props.Trim()));
                        SearchResultCollection results = seacher.FindAll();

                        if (results == null)
                        {
                            //取得失敗
                            return new Dictionary<string, string>();
                        }
                        else
                        {
                            foreach (SearchResult result in results)
                            {
                                if (result.Properties[ComConst.ATTRIBUTE_DISPLAYNAME].Count > 0)
                                {
                                    departDict.Add(result.Properties[ComConst.ATTRIBUTE_OU][0] as string, result.Properties[ComConst.ATTRIBUTE_DISPLAYNAME][0] as string);
                                }
                                else
                                {
                                    departDict.Add(result.Properties[ComConst.ATTRIBUTE_OU][0] as string, result.Properties[ComConst.ATTRIBUTE_OU][0] as string);
                                }

                            }
                            return departDict;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                logger.Error(((DirectoryServicesCOMException)ex).ExtendedErrorMessage);
                //logger.Error(username + ":セカンダリサーバでの認証失敗", ex);
                throw ex;
            }
            finally
            {
                //logger.Info("END  :" + username + ":ユーザ権限取得処理");
            }
        }


        #region 暫定対応
        /// <summary>
        /// ADの設定変更が間に合わなかったため、地域ロールを固定で置換する。
        /// 下記の地域に当てはまらない場合はそのまま返却
        /// </summary>
        /// <param name="areaCode"></param>
        /// <returns></returns>
        private static string ReplaceAreaCode(string areaCode)
        {
            switch (areaCode)
            {
                case "Tokyo": return "A";
                case "Tokai": return "C";
                case "Hokuriku": return "D";
                case "Kansai": return "E";
                case "Chugoku": return "F";
                case "Shikoku": return "G";
                case "Kyushu": return "H";
                case "Tohoku": return "I";
                case "Hokkaido": return "J";
                default: return areaCode;
            }
        }
        #endregion 暫定対応

        #region 未実装
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override string ApplicationName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override void CreateRole(string roleName)
        {
            throw new NotImplementedException();
        }

        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            throw new NotImplementedException();
        }

        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            throw new NotImplementedException();
        }

        public override string[] GetAllRoles()
        {
            throw new NotImplementedException();
        }

        public override string[] GetUsersInRole(string roleName)
        {
            throw new NotImplementedException();
        }

        public override bool IsUserInRole(string username, string roleName)
        {
            throw new NotImplementedException();
        }

        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override bool RoleExists(string roleName)
        {
            throw new NotImplementedException();
        }
        #endregion

    }
}