﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using EmpCRM.common;
using EmpCRM.Dto;

namespace EmpCRM.control
{
    public partial class DepartmentSelector : System.Web.UI.UserControl
    {
        private static string _selectedValue = string.Empty;
        //ロガー生成
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        //共通クラス
        private ComFunc m_ComFunc = new ComFunc();

        protected void Page_Load(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //ログイン状態チェック
            if (!LoginUtil.IsLogin(userRole))
            {
                Response.Redirect("error.htm", false);
            }
            else
            {
                //lblErrMessage.Text = string.Empty;

                //開始ログ出力
                logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "Page_Load");
                if (!IsPostBack)
                {
                    //初期表示処理
                    try
                    {
                        InitializeControls();
                    }
                    catch (Exception ex)
                    {
                        logger.Error("User:" + userRole.UserId + ex.Message, ex);
                    }
                }
                else
                {
                    //Postback時
                }
                //終了ログ出力
                logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "Page_Load");
            }
        }

        protected void btnDepartmentSearch_Click(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "部門検索ボタン押下時処理開始");

            if (!string.IsNullOrEmpty(txtDepartmentId.Text))
            {
                string depName = new RoleProvider().GetSectionName(txtDepartmentId.Text);
                if (depName.Equals(txtDepartmentId.Text) || string.IsNullOrEmpty(depName))
                {
                    //lblDepartmnentName.Text = "部門名が取得できませんでした。部門IDを再確認し、検索を行ってください。";
                }
                else
                {
                    //lblDepartmnentName.Text = depName;
                    Dictionary<string, string> dict = new Dictionary<string, string>();
                    dict.Add(txtDepartmentId.Text, depName);
                    createDdlByDictionary(ddlDepartment, dict);
                }
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "部門検索ボタン押下時処理終了");

        }

        protected void btnDepartmentNameSearch_Click(object sender, EventArgs e)
        {
            m_ComFunc = (ComFunc)Session[ComConst.COM_FUNC];

            UserRole userRole = Session[ComConst.SESSION_USER_ROLE] as UserRole;

            //開始ログ出力
            logger.Info(ComConst.INFO_LOG_START + userRole.UserId + "部門名検索ボタン押下時処理開始");

            if (!string.IsNullOrEmpty(txtDepartmentName.Text))
            {
                Dictionary<string, string> dictDepartment = new RoleProvider().SearchSections(txtDepartmentName.Text);
                createDdlByDictionary(ddlDepartment, dictDepartment);
            }

            //終了ログ出力
            logger.Info(ComConst.INFO_LOG_END + userRole.UserId + "部門名検索ボタン押下時処理終了");

        }

        protected void ddlDepartment_IndexChanged(object sender, EventArgs e)
        {
            _selectedValue = ddlDepartment.SelectedValue;
        }

        private void createDdlByDictionary(DropDownList ddl, Dictionary<string, string> dict)
        {
            ddl.Items.Clear();

            //空項目を追加
            ddl.Items.Add(new ListItem(string.Empty, string.Empty));

            //現在ADに存在する部門一覧をDDLに設定
            foreach (KeyValuePair<string, string> pair in dict.OrderBy(m=>m.Key))
            {
                string displayText = string.Format("{0}：{1} ", pair.Key, pair.Value);
                ddl.Items.Add(new ListItem(displayText, pair.Key));
            }
            ddlDepartment_IndexChanged(null, null);
        }

        public string GetSelectedValue()
        {
            return _selectedValue;
        }

        public void SetSelectedValue(string value)
        {
            ddlDepartment.SelectedValue = value;
        }

        public void InitializeControls()
        {
            Dictionary<string, string> dictBumon = Session[ComConst.SESSION_BUMON_MASTER] as Dictionary<string, string>;
            createDdlByDictionary(ddlDepartment, dictBumon);
        }
    }
}